# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import time
import logging
import threading
import datetime
import pandas
import types
import json
from collections import namedtuple

from django import forms

from steelscript.netprofiler.core.services import \
    Service, ServiceLocationReport
from steelscript.netprofiler.core.report import \
    Report, SingleQueryReport, TrafficTimeSeriesReport
from steelscript.netprofiler.core.filters import TimeFilter, TrafficFilter
from steelscript.common.timeutils import (parse_timedelta,
                                          timedelta_total_seconds)
from steelscript.appfwk.apps.datasource.models import \
    DatasourceTable, Column, TableQueryBase, Table
from steelscript.appfwk.apps.datasource.models import TableField
from steelscript.appfwk.apps.devices.forms import fields_add_device_selection
from steelscript.appfwk.apps.devices.devicemanager import DeviceManager
from steelscript.appfwk.apps.datasource.forms import \
    fields_add_time_selection, fields_add_resolution
from steelscript.appfwk.libs.fields import Function
from steelscript.netprofiler.core.hostgroup import HostGroupType
from steelscript.appfwk.apps.jobs import QueryComplete

logger = logging.getLogger(__name__)
lock = threading.Lock()


def _post_process_combine_filterexprs(form, id, criteria, params):
    exprs = []
    if ('netprofiler_filterexpr' in criteria and
            criteria.netprofiler_filterexpr != ''):
        exprs.append(criteria.netprofiler_filterexpr)

    field = form.get_tablefield(id)
    for parent in field.parent_keywords:
        expr = criteria[parent]
        if expr is not None and expr != '':
            exprs.append(expr)

    if len(exprs) == 0:
        val = ""
    elif len(exprs) == 1:
        val = exprs[0]
    else:
        val = "(" + ") and (".join(exprs) + ")"

    criteria['netprofiler_filterexpr'] = val


def netprofiler_hostgroup_types(form, id, field_kwargs, params):
    """ Query netprofiler for all hostgroup types. """
    netprofiler_device = form.get_field_value('netprofiler_device', id)

    if netprofiler_device == '':
        choices = [('', '<No netprofiler device>')]
    else:
        netprofiler = DeviceManager.get_device(netprofiler_device)

        choices = []

        for hgt in netprofiler.api.host_group_types.get_all():
            choices.append((hgt['name'], hgt['name']))

    field_kwargs['label'] = 'HostGroupType'
    field_kwargs['choices'] = choices


def netprofiler_hostgroups(form, id, field_kwargs, params):
    """ Query netprofiler for groups within a given hostgroup. """
    netprofiler_device = form.get_field_value('netprofiler_device', id)

    if netprofiler_device == '':
        choices = [('', '<No netprofiler device>')]
    else:
        netprofiler = DeviceManager.get_device(netprofiler_device)

        if params is not None and 'hostgroup_type' in params:
            hgt = HostGroupType.find_by_name(netprofiler,
                                             params['hostgroup_type'])
        else:
            hostgroup_type = form.get_field_value('hostgroup_type', id)

            hgt = HostGroupType.find_by_name(netprofiler,
                                             hostgroup_type)

        choices = [(group, group) for group in hgt.groups.keys()]

    field_kwargs['label'] = 'HostGroup'
    field_kwargs['choices'] = choices


def add_netprofiler_hostgroup_field(report, section, hg_type=None):
    """ Attach fields for dynamic HostGroup dropdowns to add as filter
    expressions to the report.

    This can be added for each section in a report where the added filter
    expression is desired.

    The optional ``hg_type`` argument can be either a single string or a list
    of strings for each HostGroupType.  If a single string, the
    'HostGroupType' field will be hidden and automatically filter HostGroups
    to the given HostGroupType.  If a list, the elements of the HostGroupType
    list will be fixed to those in the list; this can be helpful if certain
    HostGroupTypes may be sensitive or not applicable to the report.
    """
    # add default filter expr to extend against
    filterexpr = TableField.create(keyword='netprofiler_filterexpr')
    section.fields.add(filterexpr)

    # defaults if we are using hostgroup type field
    hg_template = '{hostgroup_type}'
    hg_parent = ['hostgroup_type']
    hg_params = None

    if hg_type is None:
        # add hostgroup types field that queries netprofiler
        field = TableField.create(
            keyword='hostgroup_type',
            label='HostGroup Type',
            obj=report,
            field_cls=forms.ChoiceField,
            parent_keywords=['netprofiler_device'],
            dynamic=True,
            pre_process_func=Function(netprofiler_hostgroup_types)
        )
        section.fields.add(field)

    elif type(hg_type) in (list, tuple):
        # add hostgroup types field that uses given list
        field = TableField.create(
            keyword='hostgroup_type',
            label='HostGroup Type',
            obj=report,
            field_cls=forms.ChoiceField,
            field_kwargs={'choices': zip(hg_type, hg_type)},
            parent_keywords=['netprofiler_device'],
        )
        section.fields.add(field)

    else:
        # no field, hardcode the given value
        hg_template = hg_type
        hg_parent = None
        hg_params = {'hostgroup_type': hg_type}

    # add hostgroup field
    field = TableField.create(
        keyword='hostgroup',
        label='HostGroup',
        obj=report,
        field_cls=forms.ChoiceField,
        parent_keywords=hg_parent,
        dynamic=True,
        pre_process_func=Function(netprofiler_hostgroups, params=hg_params)
    )
    section.fields.add(field)

    NetProfilerTable.extend_filterexpr(
        section, keyword='hg_filterexpr',
        template='hostgroup %s:{hostgroup}' % hg_template
    )


class NetProfilerTable(DatasourceTable):

    class Meta:
        proxy = True
    _query_class = 'NetProfilerQuery'

    TABLE_OPTIONS = {'groupby': None,
                     'realm': None,
                     'interface': None}

    # default field parameters
    FIELD_OPTIONS = {'duration': 60,
                     'durations': ('15 min', '1 hour',
                                   '2 hours', '4 hours', '12 hours',
                                   '1 day', '1 week', '4 weeks'),
                     'resolution': 'auto',
                     'resolutions': (('auto', 'Automatic'),
                                     '1min', '15min', 'hour', '6hour'),
                     }

    def post_process_table(self, field_options):
        resolution = field_options['resolution']
        if resolution != 'auto':
            if isinstance(resolution, int):
                res = resolution
            else:
                res = int(timedelta_total_seconds(parse_timedelta(resolution)))
            resolution = Report.RESOLUTION_MAP[res]
            field_options['resolution'] = resolution

        fields_add_device_selection(self, keyword='netprofiler_device',
                                    label='NetProfiler', module='netprofiler',
                                    enabled=True)

        duration = field_options['duration']
        if isinstance(duration, int):
            duration = "%d min" % duration

        fields_add_time_selection(self,
                                  initial_duration=duration,
                                  durations=field_options['durations'])

        fields_add_resolution(self,
                              initial=field_options['resolution'],
                              resolutions=field_options['resolutions'],
                              special_values=['auto'])
        self.fields_add_filterexpr()

    def fields_add_filterexpr(self, keyword='netprofiler_filterexpr',
                              initial=None):
        field = TableField(keyword=keyword,
                           label='NetProfiler Filter Expression',
                           help_text=('Traffic expression using NetProfiler '
                                      'Advanced Traffic Expression syntax'),
                           initial=initial,
                           required=False)
        field.save()
        self.fields.add(field)

    def fields_add_filterexprs_field(self, keyword):

        field = self.fields.get(keyword='netprofiler_filterexpr')
        field.post_process_func = Function(
            function=_post_process_combine_filterexprs
        )

        parent_keywords = set(field.parent_keywords or [])
        parent_keywords.add(keyword)
        field.parent_keywords = list(parent_keywords)
        field.save()

        return field

    @classmethod
    def extend_filterexpr(cls, obj, keyword, template):

        field = obj.fields.get(keyword='netprofiler_filterexpr')
        field.post_process_func = Function(
            function=_post_process_combine_filterexprs
        )

        TableField.create(
            keyword=keyword, obj=obj, hidden=True,
            post_process_template=template)

        parent_keywords = set(field.parent_keywords or [])
        parent_keywords.add(keyword)
        field.parent_keywords = list(parent_keywords)
        field.save()


class NetProfilerTimeSeriesTable(NetProfilerTable):
    class Meta:
        proxy = True

    TABLE_OPTIONS = {'groupby': 'time',
                     'realm': 'traffic_overall_time_series',
                     'interface': None,
                     'limit': None}


class NetProfilerGroupbyTable(NetProfilerTable):
    class Meta:
        proxy = True

    TABLE_OPTIONS = {'groupby': None,
                     'realm': 'traffic_summary',
                     'interface': None,
                     'limit': None}


class NetProfilerQuery(TableQueryBase):

    def _prepare_report_args(self):
        class Args(object):
            pass
        args = Args()

        criteria = self.job.criteria

        if criteria.netprofiler_device == '':
            logger.debug('%s: No netprofiler device selected' % self.table)
            self.job.mark_error("No NetProfiler Device Selected")
            return False

        args.profiler = DeviceManager.get_device(criteria.netprofiler_device)

        args.columns = [col.name for col
                        in self.table.get_columns(synthetic=False)]

        args.sortcol = None
        if self.table.sortcols is not None:
            args.sortcol = self.table.sortcols[0]

        args.timefilter = TimeFilter(start=criteria.starttime,
                                     end=criteria.endtime)

        logger.info("Running NetProfiler table %d report for timeframe %s" %
                    (self.table.id, str(args.timefilter)))

        if ('datafilter' in criteria) and (criteria.datafilter is not None):
            args.datafilter = criteria.datafilter.split(',')
        else:
            args.datafilter = None

        args.trafficexpr = TrafficFilter(
            self.job.combine_filterexprs(exprs=criteria.netprofiler_filterexpr)
        )

        # Incoming criteria.resolution is a timedelta
        logger.debug('NetProfiler report got criteria resolution %s (%s)' %
                     (criteria.resolution, type(criteria.resolution)))
        if criteria.resolution != 'auto':
            rsecs = int(timedelta_total_seconds(criteria.resolution))
            args.resolution = Report.RESOLUTION_MAP[rsecs]
        else:
            args.resolution = 'auto'

        logger.debug('NetProfiler report using resolution %s (%s)' %
                     (args.resolution, type(args.resolution)))

        args.limit = (self.table.options.limit
                      if hasattr(self.table.options, 'limit') else None)

        args.centricity = 'int' if self.table.options.interface else 'hos'

        return args

    def _wait_for_data(self, report, minpct=0, maxpct=100):
        criteria = self.job.criteria
        done = False
        logger.info("Waiting for report to complete")
        while not done:
            time.sleep(0.5)
            with lock:
                s = report.status()

            logger.debug('Status: XXX %s' % str(s))
            pct = int(float(s['percent']) * ((maxpct - minpct)/100.0) + minpct)
            self.job.mark_progress(progress=pct)
            done = (s['status'] == 'completed')

        # Retrieve the data
        with lock:
            data = report.get_data()

            tz = criteria.starttime.tzinfo
            # Update criteria
            query = report.get_query_by_index(0)
            criteria.starttime = (datetime.datetime
                                  .utcfromtimestamp(query.actual_t0)
                                  .replace(tzinfo=tz))
            criteria.endtime = (datetime.datetime
                                .utcfromtimestamp(query.actual_t1)
                                .replace(tzinfo=tz))

        self.job.safe_update(actual_criteria=criteria)
        return data

    def run(self):
        """ Main execution method
        """
        args = self._prepare_report_args()

        with lock:
            report = SingleQueryReport(args.profiler)
            report.run(
                realm=self.table.options.realm,
                groupby=args.profiler.groupbys[self.table.options.groupby],
                centricity=args.centricity,
                columns=args.columns,
                timefilter=args.timefilter,
                trafficexpr=args.trafficexpr,
                data_filter=args.datafilter,
                resolution=args.resolution,
                sort_col=args.sortcol,
                sync=False,
                limit=args.limit
            )

        data = self._wait_for_data(report)

        if self.table.rows > 0:
            data = data[:self.table.rows]

        logger.info("Report %s returned %s rows" % (self.job, len(data)))
        return QueryComplete(data)


#
# Traffic Time Series
#
# Timeseries report with criteria per columns, as opposed to just a time series
#
class NetProfilerTrafficTimeSeriesTable(NetProfilerTable):

    class Meta:
        proxy = True

    TABLE_OPTIONS = {'base': None,
                     'groupby': None,
                     'col_criteria': None,
                     'interface': None,
                     'top_n': None,
                     'include_other': False}

    _query_class = 'NetProfilerTrafficTimeSeriesQuery'

    @classmethod
    def process_options(cls, table_options):
        # handle direct id's, table references, or table classes
        # from tables option and transform to simple table id value
        table_options['base'] = Table.to_ref(table_options['base'])
        return table_options

    def post_process_table(self, field_options):
        super(NetProfilerTrafficTimeSeriesTable, self).post_process_table(
            field_options)

        if self.options.top_n is None:
            # If not top-n, the criteria field 'query_columns' must
            # either be a string or an array of column definitions
            # (a string is simply parsed as json to the latter).
            #
            # This criteria field must resolve to an array of
            # field definitions, one per column to be queried
            #
            # An array of column defintions looks like the following:
            #   [ {'name': <name>, 'label': <name>, 'json': <json>},
            #     {'name': <name>, 'label': <name>, 'json': <json>},
            #     ... ]
            #
            # Each element corresponds to a column requested.  <name> is
            # used as the Column.name, <label> is for the Column.label
            # and json is what is passed on to NetProfiler in the POST
            # to create the report
            #
            TableField.create(keyword='query_columns',
                              label='Query columns',
                              obj=self)


TSQ_Tuple = namedtuple('TSQ_Tuple', ['groupby', 'columns', 'parser'])


class NetProfilerTrafficTimeSeriesQuery(NetProfilerQuery):

    # Dictionary of config for running time-series/top-n queries for a
    # requested groupby.  The components are:
    #
    #    groupby:  the groupby to use for the time-series query, usually
    #              just the plural form of the standard NetProfiler groupby
    #
    #    columns:  the key column(s) to ask for as part of the query
    #
    #    parser:   the name of the row parsing function that takes a row
    #              and converts the row/keys into the necessary form
    #              as required by the time-series groupby report (in run())
    #
    CONFIG = {
        'port':
            TSQ_Tuple('ports', ['protoport_parts'], 'parse_port'),
        'application':
            TSQ_Tuple('applications', ['app_name', 'app_raw'], 'parse_app'),
        'host_group':
            TSQ_Tuple('host_groups', ['group_name'], 'parse_host_group'),
    }

    @classmethod
    def parse_app(cls, row):
        app_name = row[0]
        app_raw = row[1]

        return {'name': app_name,
                'label': app_name,
                'json': {'code': app_raw}}

    @classmethod
    def parse_port(cls, row):
        proto, port = row[0].split('|')

        return {'name': '%s%s' % (proto, port),
                'label': '%s/%s' % (proto, port),
                'json': {'name': '%s/%s' % (proto, port)}}

    @classmethod
    def parse_host_group(cls, row):
        group_name = row[0]

        return {'name': group_name,
                'label': group_name,
                'json': {'name': group_name}}

    # Run a SingleQueryReport based on the requested groupby and
    # return a list of column definitions that will be passed
    # on to the TrafficTimeSeriesReport query_columns argument
    def run_top_n(self, config, args, base_col, minpct, maxpct):
        columns = config.columns + [base_col.name]
        with lock:
            report = SingleQueryReport(args.profiler)
            report.run(
                realm='traffic_summary',
                groupby=args.profiler.groupbys[self.table.options.groupby],
                columns=columns,
                timefilter=args.timefilter,
                trafficexpr=args.trafficexpr,
                resolution=args.resolution,
                sort_col=base_col.name,
                sync=False
                )

        rows = self._wait_for_data(report, minpct=minpct, maxpct=maxpct)

        defs = []
        parser = getattr(self, config.parser)

        for row in rows[:int(self.table.options.top_n)]:
            defs.append(parser(row))

        return defs

    # This is the main run method and will run up to 3 reports
    #
    #   1. Top-N report -- if table.options.top_n is specified, this report
    #      drives what columns are requested
    #
    #   2. TrafficTimeSeriesReport - a time-series report with one column
    #      per requested criteria.
    #
    #   3. Other report -- a time-series report showing all traffic, use to
    #      compute "other" if table.options.include_other
    #
    def run(self):
        args = self._prepare_report_args()
        base_table = Table.from_ref(self.table.options.base)
        base_col = base_table.get_columns()[0]

        # only calculate other when we aren't filtering data
        include_other = self.table.options.include_other
        if self.job.criteria.netprofiler_filterexpr:
            include_other = False

        if self.table.options.groupby not in self.CONFIG:
            raise ValueError('not supported for groupby=%s' %
                             self.table.options.groupby)

        config = self.CONFIG[self.table.options.groupby]

        # num_reports / cur_report are used to compute min/max pct
        num_reports = (1 +
                       (1 if self.table.options.top_n else 0) +
                       (1 if include_other else 0))
        cur_report = 0

        if self.table.options.top_n:
            # Run a top-n report to drive the criteria for each column
            query_column_defs = self.run_top_n(config, args, base_col,
                                               minpct=0,
                                               maxpct=(100/num_reports))
            cur_report += 1
        else:
            query_column_defs = self.job.criteria.query_columns
            if isinstance(query_column_defs, types.StringTypes):
                query_column_defs = json.loads(query_column_defs)

        query_columns = [col['json'] for col in query_column_defs]

        with lock:
            report = TrafficTimeSeriesReport(args.profiler)
            columns = [args.columns[0], base_col.name]
            logger.info("Query Columns: %s" % str(query_columns))

            if self.table.options.groupby == 'host_group':
                host_group_type = 'ByLocation'
            else:
                host_group_type = None

            report.run(
                centricity=args.centricity,
                columns=columns,
                timefilter=args.timefilter,
                trafficexpr=args.trafficexpr,
                resolution=args.resolution,
                sync=False,
                host_group_type=host_group_type,
                query_columns_groupby=config.groupby,
                query_columns=query_columns
            )

        data = self._wait_for_data(report,
                                   minpct=cur_report * (100/num_reports),
                                   maxpct=(cur_report + 1) * (100/num_reports))
        cur_report += 1

        df = pandas.DataFrame(data,
                              columns=(['time'] + [col['name'] for
                                                   col in query_column_defs]))

        # Create ephemeral columns for all the data based
        # on the related base table
        for col in query_column_defs:
            Column.create(self.job.table, col['name'], col['label'],
                          ephemeral=self.job, datatype=base_col.datatype,
                          formatter=base_col.formatter)

        if include_other:
            # Run a separate timeseries query with no column filters
            # to get "totals" then use that to compute an "other" column

            with lock:
                report = SingleQueryReport(args.profiler)
                report.run(
                    realm='traffic_overall_time_series',
                    groupby=args.profiler.groupbys['time'],
                    columns=columns,
                    timefilter=args.timefilter,
                    trafficexpr=args.trafficexpr,
                    resolution=args.resolution,
                    sync=False
                )

            totals = self._wait_for_data(report,
                                         minpct=cur_report * (100/num_reports),
                                         maxpct=(cur_report + 1) * (100/num_reports))

            df = df.set_index('time')
            df['subtotal'] = df.sum(axis=1)
            totals_df = (pandas.DataFrame(totals, columns=['time', 'total'])
                         .set_index('time'))

            df = df.merge(totals_df, left_index=True, right_index=True)
            df['other'] = df['total'] = df['subtotal']
            colnames = ['time'] + [col['name'] for col in query_column_defs] + ['other']

            # Drop the extraneous total and subtotal columns
            df = (df.reset_index().ix[:, colnames])

            Column.create(self.job.table, 'other', 'Other',
                          ephemeral=self.job, datatype=base_col.datatype,
                          formatter=base_col.formatter)

        logger.info("Report %s returned %s rows" % (self.job, len(df)))
        return QueryComplete(df)


#
# Service reports
#

class NetProfilerServiceByLocTable(DatasourceTable):

    class Meta:
        proxy = True
    _query_class = 'NetProfilerServiceByLocQuery'

    # rgb - red/yellow/green, if True return string values
    #       instead of numbers
    TABLE_OPTIONS = {'rgb': True}

    FIELD_OPTIONS = {'duration': '15min',
                     'durations': ('15 min', '1 hour',
                                   '2 hours', '4 hours', '12 hours',
                                   '1 day', '1 week', '4 weeks'),
                     }

    def post_process_table(self, field_options):
        fields_add_device_selection(self, keyword='netprofiler_device',
                                    label='NetProfiler', module='netprofiler',
                                    enabled=True)

        duration = field_options['duration']

        fields_add_time_selection(self,
                                  initial_duration=duration,
                                  durations=field_options['durations'])


class NetProfilerServiceByLocQuery(TableQueryBase):

    def run(self):
        """ Main execution method
        """
        criteria = self.job.criteria

        if criteria.netprofiler_device == '':
            logger.debug('%s: No netprofiler device selected' % self.table)
            self.job.mark_error("No NetProfiler Device Selected")
            return False

        profiler = DeviceManager.get_device(criteria.netprofiler_device)
        report = ServiceLocationReport(profiler)

        tf = TimeFilter(start=criteria.starttime,
                        end=criteria.endtime)

        logger.info(
            'Running NetProfilerServiceByLocTable %d report for timeframe %s' %
            (self.table.id, str(tf)))

        with lock:
            report.run(timefilter=tf, sync=False)

        done = False
        logger.info("Waiting for report to complete")
        while not done:
            time.sleep(0.5)
            with lock:
                s = report.status()

            self.job.mark_progress(progress=int(s['percent']))
            done = (s['status'] == 'completed')

        # Retrieve the data
        with lock:
            data = report.get_data()
            query = report.get_query_by_index(0)

            tz = criteria.starttime.tzinfo
            # Update criteria
            criteria.starttime = (datetime.datetime
                                  .utcfromtimestamp(query.actual_t0)
                                  .replace(tzinfo=tz))
            criteria.endtime = (datetime.datetime
                                .utcfromtimestamp(query.actual_t1)
                                .replace(tzinfo=tz))

        self.job.safe_update(actual_criteria=criteria)

        if len(data) == 0:
            return QueryComplete(None)

        # Add ephemeral columns for everything
        Column.create(self.job.table, 'location', 'Location',
                      ephemeral=self.job, datatype='string')
        for k in data[0].keys():
            if k == 'location':
                continue

            Column.create(self.job.table, k, k,
                          ephemeral=self.job, datatype='string',
                          formatter='rvbd.formatHealth')

        df = pandas.DataFrame(data)

        if self.job.table.options.rgb:
            state_map = {Service.SVC_NOT_AVAILABLE: 'gray',
                         Service.SVC_DISABLED: 'gray',
                         Service.SVC_INIT: 'gray',
                         Service.SVC_NORMAL: 'green',
                         Service.SVC_LOW: 'yellow',
                         Service.SVC_MED: 'yellow',
                         Service.SVC_HIGH: 'red',
                         Service.SVC_NODATA: 'gray'}

            df = df.replace(state_map.keys(),
                            state_map.values())

        return QueryComplete(df)



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.netprofiler.core.app import NetProfilerApp
from steelscript.netprofiler.core.report import WANSummaryReport, WANTimeSeriesReport
from steelscript.netprofiler.core.filters import TimeFilter

import sys
import optparse
import cStringIO as StringIO

# suppress warnings from pandas 0.11
import warnings
warnings.simplefilter(action="ignore", category=FutureWarning)


class WANReportApp(NetProfilerApp):

    def add_options(self, parser):
        super(WANReportApp, self).add_options(parser)
        group = optparse.OptionGroup(parser, "Device List Options")
        group.add_option('--device-address', dest='device_address', default=None,
                         help='IP address for WAN device')
        group.add_option('--device-name', dest='device_name', default=None,
                         help='Text of device name to search for (simple search only,'
                              ' no regular expressions')
        group.add_option('--lan-address', dest='lan_address', default=None,
                         help='LAN interface address')
        group.add_option('--wan-address', dest='wan_address', default=None,
                         help='WAN interface address')
        parser.add_option_group(group)

        group = optparse.OptionGroup(parser, "Filter Options")
        group.add_option('--timefilter', dest='timefilter', default='last 1 hour',
                         help='Time range to analyze (defaults to "last 1 hour")')
        parser.add_option_group(group)

        group = optparse.OptionGroup(parser, "Report Type Options (choose one)")
        group.add_option('--summary', dest='summary', default=False, action='store_true',
                         help='Generate Summary report of WAN address')
        group.add_option('--time-series', dest='time_series', default=False, action='store_true',
                         help='Generate Time Series report of WAN address')
        parser.add_option_group(group)

        group = optparse.OptionGroup(parser, "Output options")
        group.add_option('--inbound', dest='out_inbound', default=False, action='store_true',
                         help='Print inbound statistics')
        group.add_option('--outbound', dest='out_outbound', default=False, action='store_true',
                         help='Print outbound statistics')
        group.add_option('--combined', dest='out_combined', default=False, action='store_true',
                         help='Print combined inbound/outbound statistics')
        group.add_option('--csv', dest='as_csv', default=False, action='store_true',
                         help='Return values in CSV format instead of tabular')

        parser.add_option_group(group)

    def validate_args(self):
        """ Ensure either wan-address or wan-device-name chosen
        """
        super(WANReportApp, self).validate_args()

        if (not self.options.device_address and
                not self.options.device_name and
                not (self.options.lan_address and self.options.wan_address)):
            self.parser.error('Either device-address, device-name or '
                              'both lan-address and wan-address required')
        elif not self.options.summary and not self.options.time_series:
            self.parser.error('Either summary or time_series option required')
        elif not any([self.options.out_inbound,
                      self.options.out_outbound,
                      self.options.out_combined]):
            self.parser.error('Choose at least one output option: '
                              '--inbound, --outbound, --combined')

    def print_data(self, data, header):
        if self.options.as_csv:
            f = StringIO.StringIO()
            data.to_csv(f, header=True)
            print f.getvalue()
        else:
            print header
            print data

    def main(self):
        self.ip_address = None
        self.lan_address = None
        self.wan_address = None
        self.timefilter = TimeFilter.parse_range(self.options.timefilter)

        if self.options.wan_address and self.options.lan_address:
            self.ip_address = self.options.wan_address.split(':')[0]
            self.lan_address = [self.options.lan_address]
            self.wan_address = [self.options.wan_address]
        elif self.options.device_name:
            name = self.options.device_name
            devices = self.netprofiler.api.devices.get_all()
            for d in devices:
                if name.lower() in d['name'].lower():
                    self.ip_address = d['ipaddr']
                    break
            else:
                print 'Device %s cannot be found in NetProfiler device list' % name
                print 'Try specifying the name differently or use an IP address'
                sys.exit(1)
        else:
            self.ip_address = self.options.device_address

        if self.options.summary:
            self.columns = ['device',
                            'avg_bytes',
                            'total_bytes']
            self.groupby = 'dev'
            ReportClass = WANSummaryReport
        else:
            # Time Series report
            self.columns = ['time',
                            'avg_bytes',
                            'total_bytes']
            self.groupby = None
            ReportClass = WANTimeSeriesReport

        with ReportClass(self.netprofiler) as report:
            if not self.lan_address:
                # query for the interfaces
                self.lan_address, self.wan_address = report.get_interfaces(self.ip_address)

            if self.options.out_inbound or self.options.out_combined:
                # inbound
                report.run(self.lan_address, self.wan_address, 'inbound', columns=self.columns,
                           groupby=self.groupby, timefilter=self.timefilter, resolution='auto')
                inbound = report.get_data(as_list=False)

                if self.options.out_inbound:
                    header = 'Inbound traffic:'
                    self.print_data(inbound, header)

            if self.options.out_outbound or self.options.out_combined:
                # outbound
                report.run(self.lan_address, self.wan_address, 'outbound', columns=self.columns,
                           groupby=self.groupby, timefilter=self.timefilter, resolution='auto')
                outbound = report.get_data(as_list=False)

                if self.options.out_outbound:
                    header = 'Outbound traffic:'
                    self.print_data(outbound, header)

            if self.options.out_combined:
                header = 'Combined Inbound/Outbound traffic:'
                total = inbound + outbound
                self.print_data(total, header)


if __name__ == '__main__':
    WANReportApp().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

"""
The Host Group module provides an interface for manipulating host group types
and their host groups and hosts.
"""

from steelscript.common.exceptions import RvbdException, RvbdHTTPException
import logging

# Examples:
#
# >>> byloc = HostGroupType.find_by_name(netprofiler, 'ByLocation')
#
# >>> byloc.group['sanfran']
# <HostGroup 'sanfran'>
#
# >>> byloc.group['sanfran'].get()
#   ['10.99.1/24']
#
#   >>> byloc.group['sanfran'].add('10.99.2/24')
#   ['10.99.1/24', '10.99.2/24']
#
#   >>> byloc.save()
#

logger = logging.getLogger(__name__)


class HostGroupType(object):
    """ Convenience class to allow easy access to host group types.

    Use this class to create new and access existing host group types.
    Within a host group type you can add and remove it's :class:`HostGroups`
    and their members. All changes to host group types will be *local*
    until :func:`save` is called.

    Example **accessing** an existing :class:`HostGroupType` and
    adding a new :class:`HostGroup` member to it::

        >>> byloc = HostGroupType.find_by_name(netprofiler, 'ByLocation')
        >>> sanfran = byloc.group['sanfran']
        <HostGroup 'sanfran'>
        >>> sanfran.get()
        ['10.99.1/24']
        >>> sanfran.add('10.99.2/24')
        ['10.99.1/24', '10.99.2/24']
        >>> byloc.save()


    Example **creating** a new :class:`HostGroupType`, :class:`HostGroup`,
    and group member::

        >>> by_region = HostGroupType.create(netprofiler, 'ByRegion')
        >>> north_america = HostGroup(by_region, 'north_america')
        <HostGroup 'north_america'>
        >>> north_america.get()
        []
        >>> north_america.add(['10.99.1/24', '10.99.2/24'])
        ['10.99.1/24', '10.99.2/24']
        >>> by_region.save()

    """

    def __init__(self, netprofiler, id):
        """
        :class:`HostGroupType` should not be instantiated directly,
        instead use :func:`create` or :func:`find_by_name`.
        """
        # Host group id
        self.id = id
        # NetProfiler
        self.netprofiler = netprofiler

        # Properties filled in by load or by create
        self.name = ""
        self.favorite = ""
        self.description = ""

        # Array of cidr/name config items
        self.config = []

        # Dictionary of HostGroup entries by name
        self.groups = {}

    @classmethod
    def find_by_name(cls, netprofiler, name):
        """Find and load a host group type by name."""
        type_id = HostGroupType._find_id(netprofiler, name)
        host_group_type = HostGroupType(netprofiler, type_id)
        host_group_type.load()
        return host_group_type

    @classmethod
    def create(cls, netprofiler, name, favorite=False, description=''):
        """Create a new hostgroup type.

        :param Netprofiler netprofiler: The Netprofiler you are using.
        :param str name: The name of the new :class:`HostGroupType`.
        :param bool favorite: if True, this type will be listed as a favorite.
        :param str description: The hostgroup type's description.

        The new host group type will be created on the NetProfiler
        when :func:`save` is called.

        """
        # host_group_type.id is set to None until it gets saved
        host_group_type = HostGroupType(netprofiler, None)
        host_group_type.name = name
        host_group_type.favorite = favorite
        host_group_type.description = description
        return host_group_type

    def load(self):
        """Load settings and groups."""
        if self.id is None:
            raise RvbdException('Type: "{0}" has not yet been saved to the '
                                'Netprofiler, so there is nothing to load. '
                                'Call $host_group_type.save() first to save it.'
                                .format(self.name))
        info = self.netprofiler.api.host_group_types.get(self.id)
        self.name = info['name']
        self.favorite = info['favorite']
        self.description = info['description']

        try:
            self.config = self.netprofiler.api.host_group_types.get_config(self.id)
        except RvbdHTTPException as e:
            # When you call get_config a RESOURCE_NOT_FOUND error is raised if
            # the config of that type is empty, even if the host group type
            # exists. Because of this we except that error and move on with load
            if e.error_id == 'RESOURCE_NOT_FOUND':
                logger.debug('RESOURCE_NOT_FOUND exception raised because the '
                             'config is empty. It was excepted because we '
                             'still want the HostGroupType.')
                self.config = []
                self.groups = {}
            else:
                raise e

        # Get the groups, we will need to reformat the output to fit our dict.
        for entry in self.config:
            if entry['name'] not in self.groups:
                # Note -- only need to create a HostGroup for the *first*
                # entry found for a given name, as HostGroup doesn't actually
                # store data, it reference back to this objects 'config'
                # property
                HostGroup(self, entry['name'])

    def save(self):
        """Save settings and groups.

        If this is a new host group type, it will be created.
        """

        # If this is a new HostGroupType, then create it
        if self.id is None:
            type_info = self.netprofiler.api.host_group_types.create(
                self.name, self.description, self.favorite, self.config)
            self.id = type_info['id']
            logger.debug("New HostGroupType created with Name: {0} and ID: {1}"
                         .format(self.name, self.id))
            return
        # Otherwise just set the preexisting HostGroupType with the new info
        self.netprofiler.api.host_group_types.set(self.id, self.name,
                                                  self.description,
                                                  self.favorite, self.config)

    def delete(self):
        """Delete this host group type and all groups."""
        if self.id is None:
            raise RvbdException('Type: "{0}" has not yet been saved to the '
                                'Netprofiler, so there is nothing to delete. '
                                'Call $host_group_type.save() first to save it.'
                                .format(self.name))
        self.netprofiler.api.host_group_types.delete(self.id)
        self.id = None

    def _add_host_group(self, new_host_group):
        """ Add a new host group to groups dictionary.

        :param new_host_group: the new HostGroup to be added

        """
        if new_host_group.name in self.groups.keys():
            raise RvbdException('Host group: "{0}" already exists.'
                                .format(new_host_group.name))

        self.groups[new_host_group.name] = new_host_group

    @classmethod
    def _find_id(cls, netprofiler, name):
        # Get the ID of the host type specified by name
        host_types = netprofiler.api.host_group_types.get_all()
        target_type_id = None
        for host_type in host_types:
            if name == host_type['name']:
                target_type_id = host_type['id']
                break
        # If target_type_id is still None, then we didn't find that host
        if target_type_id is None:
            raise RvbdException('{0} is not a valid type name '
                                'for this netprofiler'.format(name))
        return target_type_id


class HostGroup(object):
    # This is a convenience class to work with a single host
    # group definition.  We don't want to store the actual
    # host group definitions here because as much as possible
    # we need to preserve the order of *all* config items
    # across host group definitions because of precedence
    #
    # As such, all operations like add/remove/clear
    # operate on hostgrouptype.config

    def __init__(self, hostgrouptype, name):
        """New object representing a host group by name.

        The new :class:`HostGroup` will be automatically added to the
        provided :class:`HostGroupType` and can be accessed with::

            host_group_type.groups['group_name']

        """
        if not isinstance(name, basestring):
            raise RvbdException("This host group's name is not a string.")
        self.host_group_type = hostgrouptype
        self.name = name
        self.host_group_type._add_host_group(self)

    def add(self, cidrs, prepend=False, keep_together=True,
            replace=False):
        """Add a CIDR to this definition.

        :param string/list cidrs: CIDR or list of CIDRS to add to this
            host group

        :param bool prepend: if True, prepend instead of append

        :param bool keep_together: if True, place
            new entries near the other entries in this
            hostgroup.  If False, append/prepend to
            relative to the entire list.

        :param bool replace: if True, replace existing
            config entries for this host group with ``cidrs``

        """
        # prepend, keep_together define where a new entry will
        # show up in hostgrouptype.config
        #
        # config = [ {'name': 'Boston',  'cidr': '10.99.1/24'},
        #            {'name': 'Boston',  'cidr': '10.99.2/24'},
        #            {'name': 'SanFran', 'cidr': '10.99.3/24'},
        #            {'name': 'SanFran', 'cidr': '10.99.4/24'},
        #            {'name': 'NewYork', 'cidr': '10.99.5/24'},
        #            {'name': 'NewYork', 'cidr': '10.99.6/24'},
        #
        # sanfran=HostGroup(hostgrouptype, 'SanFran')
        #
        # The following shows the 4 variations of prepend/keep_together:
        #
        # sanfran.add('10.99.7/24', prepend=False, keep_together=True)
        #   - new entry between 10.99.4 and 10.99.5
        #
        # sanfran.add('10.99.7/24', prepend=True, keep_together=True)
        #   - new entry between 10.99.2 and 10.99.3
        #
        # sanfran.add('10.99.7/24', prepend=False, keep_together=False)
        #   - new entry at the back (after 10.99.6)
        #
        # sanfran.add('10.99.7/24', prepend=True, keep_together=False)
        #   - new entry at the front (before 10.99.1)
        #
        # If replace=True, the position will be chosen based on entries
        # before being deleted.  If not entries exist, it will be
        # as if keep_together=False (so front or back)

        if isinstance(cidrs, basestring):
            cidrs = [cidrs]

        if replace:
            self.clear()

        # Format the cidrs to be in the correct format for the config
        old_config = self.host_group_type.config
        new_config = []
        for cidr in cidrs:
            new_config.append({'cidr': cidr, 'name': self.name})

        # Add the new_config in the correct location
        if keep_together and len(old_config) > 0:
            index = [i for i, j in enumerate(old_config)
                     if j['name'] == self.name]
            if len(index) != 0:
                if prepend:
                    self.host_group_type.config[index[0]:index[0]] = new_config
                else:
                    self.host_group_type.config[index[-1]+1:index[-1]+1] = new_config
                return
        if prepend:
            self.host_group_type.config[0:0] = new_config
        else:
            self.host_group_type.config.extend(new_config)

    def remove(self, cidrs):
        """Remove a CIDR from this host group.

        :param string/list cidrs: CIDR or list of CIDRS to remove from
            this host group

        """
        if isinstance(cidrs, basestring):
            cidrs = [cidrs]
        self.host_group_type.config = filter(
            lambda a: a['cidr'] not in cidrs or a['name'] != self.name,
            self.host_group_type.config)

    def clear(self):
        """Clear all definitions for this host group."""
        self.host_group_type.config = filter(lambda a: a['name'] != self.name,
                                             self.host_group_type.config)

    def get(self):
        """Return a list of CIDRs assigned to this host group."""
        return [i['cidr'] for i in self.host_group_type.config
                if i['name'] == self.name]



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.appfwk.apps.report.models import Report
import steelscript.appfwk.apps.report.modules.yui3 as yui3

from steelscript.netprofiler.appfwk.datasources.netprofiler import \
    NetProfilerTimeSeriesTable, NetProfilerGroupbyTable, \
    add_netprofiler_hostgroup_field
#
# NetProfiler report
#

report = Report.create("NetProfiler HostGroup Report", position=10,
                       field_order=['netprofiler_device', 'endtime',
                                    'duration', 'resolution', 'hostgroup_type',
                                    'hostgroup', 'netprofiler_filterexpr'])

section = report.add_section()

add_netprofiler_hostgroup_field(report, section)

# Define a Overall TimeSeries showing Avg Bytes/s
p = NetProfilerTimeSeriesTable.create('ts-overall',
                                      duration=60, resolution="1min")

p.add_column('time', 'Time', datatype='time', iskey=True)
p.add_column('avg_bytes', 'Avg Bytes/s', units='B/s')

report.add_widget(yui3.TimeSeriesWidget, p, "Overall Traffic", width=12)

# Define a Pie Chart for top ports
p = NetProfilerGroupbyTable.create('ports-bytes',
                                   groupby='port_group', duration=60)

p.add_column('portgroup', 'Port Group', iskey=True)
p.add_column('avg_bytes', 'Avg Bytes/s', units='B/s', sortdesc=True)

report.add_widget(yui3.PieWidget, p, "Port Groups by Avg Bytes")

# Define a Bar Chart for application ports
p = NetProfilerGroupbyTable.create('application-bytes',
                                   groupby='application_port', duration=60)

p.add_column('protoport_name', 'Application Port', iskey=True)
p.add_column('avg_bytes', 'Avg Bytes/s', units='B/s', sortdesc=True)
report.add_widget(yui3.BarWidget, p, "Application Ports by Avg Bytes")

# Define a TimeSeries showing Avg Bytes/s for tcp/80
p = NetProfilerTimeSeriesTable.create('ts-tcp80', duration=60,
                                      filterexpr='tcp/80', cacheable=False)

p.add_column('time', 'Time', datatype='time', iskey=True)
p.add_column('avg_bytes', 'Avg Bytes/s', units='B/s')
p.add_column('avg_bytes_rtx', 'Avg Retrans Bytes/s', units='B/s')

report.add_widget(yui3.TimeSeriesWidget, p, "Bandwidth for tcp/80",
                  altaxis=['avg_bytes_rtx'])

# Define a TimeSeries showing Avg Bytes/s for tcp/443
p = NetProfilerTimeSeriesTable.create('ts-tcp443',
                                      duration=60, filterexpr='tcp/443')

p.add_column('time', 'Time', datatype='time', iskey=True)
p.add_column('avg_bytes', 'Avg Bytes/s', units='B/s')
p.add_column('avg_bytes_rtx', 'Avg Retrans Bytes/s', units='B/s')

report.add_widget(yui3.TimeSeriesWidget, p, "Bandwidth for tcp/443")



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

# When NetProfiler runs a report and generates "dynamic" or "ephemeral"
# columns, they are given an id greater than or equal to 200000
EPHEMERAL_COLID = 200000

realms = ['traffic_summary',
          'traffic_overall_time_series',
          'traffic_flow_list',
          'identity_list',
          'msq'
          ]

centricities = ['hos', 'int']

# full list of groupbys from recent API query
# includes originally 'excluded' items commented out
groupbys = dict([
#                    ('alert_level', 'alt'),
#                    ('analytic', 'anl'),
#                    ('analytic_category', 'anc'),
                    ('application', 'app'),
                    ('application_port', 'apt'),
                    ('application_protoport_qos', 'apq'),
                    ('device', 'dev'),
#                    ('event', 'evt'),
#                    ('flow', 'flw'),
                    ('group_pair_protoport', 'gpr'),
                    ('host', 'hos'),
                    ('host_and_vtep_pair', 'vhp'),
                    ('host_group', 'gro'),
                    ('host_group_pair', 'gpp'),
                    ('host_pair', 'hop'),
                    ('host_pair_protoport', 'hpr'),
                    ('interface', 'ifc'),
#                    ('interface_group', 'ifg'),
                    ('interface_qos', 'ifq'),
                    ('ip_mac', 'ipm'),
                    ('ip_mac_pair', 'ipp'),
                    ('ip_mac_pair_protoport', 'ipr'),
#                    ('metric', 'mtc'),
                    ('peer', 'per'),
                    ('peer_group', 'pgp'),
                    ('peer_ip_mac', 'pip'),
#                    ('policy', 'pol'),
                    ('port', 'por'),
                    ('port_group', 'pgr'),
                    ('protocol', 'pro'),
                    ('segment', 'seg'),
#                    ('service', 'svc'),
#                    ('service_component', 'scm'),
#                    ('service_full_aggregation', 'saf'),
#                    ('service_location_aggregation', 'sal'),
                    ('service_location_metric_aggregation', 'slm'),
#                    ('service_location_summary', 'sll'),
#                    ('service_segment', 'ssg'),
#                    ('service_segment_aggregation', 'sas'),
                    ('time', 'tim'),
                    ('time_host_user', 'thu'),
#                    ('topology_summary', 'tpl'),
                    ('total', 'mzt'),
#                    ('vtep', 'vtp'),
                    ('vtep_pair', 'vpa'),
#                    ('vtep_peer', 'vpe'),
                    ('vxlan', 'vxl')
                ])



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.common.api_helpers import APIVersion


class APIGroup(object):
    """Wrapper for API functions
    """
    def __init__(self, uri_prefix, service):
        self.uri_prefix = uri_prefix
        self.service = service


class API1Group(APIGroup):
    def _json_request(self, urlpath, method='GET', data=None, params=None):
        """Issue the given API request via JSON
        """
        return self.service.conn.json_request(method, self.uri_prefix + urlpath,
                                              body=data, params=params)


class Common(API1Group):
    def __init__(self, *args, **kwargs):
        super(Common, self).__init__(*args, **kwargs)

    def info(self):
        return self._json_request('/info.json')

    def logout(self):
        return self._json_request('/logout.json')


class Report(API1Group):
    def __init__(self, *args, **kwargs):
        super(Report, self).__init__(*args, **kwargs)
        self.columns_cache = dict()
        self.realms_cache = None
        self.centricities_cache = None
        self.groupbys_cache = None
        self.areas_cache = None

    def realms(self, force=False):
        if not self.realms_cache or force:
            self.realms_cache = self._json_request('/realms.json')
        return self.realms_cache

    def centricities(self, force=False):
        if not self.centricities_cache or force:
            self.centricities_cache = self._json_request('/centricities.json')
        return self.centricities_cache

    def groupbys(self, force=False):
        if not self.groupbys_cache or force:
            self.groupbys_cache = self._json_request('/group_bys.json')
        return self.groupbys_cache

    def columns(self, realm, centricity, group_by, force=False):
        params = {}
        params['group_by'] = group_by
        params['centricity'] = centricity
        params['realm'] = realm
        key = str(realm) + str(centricity) + str(group_by)
        if key not in self.columns_cache or force:
            self.columns_cache[key] = self._json_request('/columns.json',
                                                         params=params)
        return self.columns_cache[key]

    def areas(self, force=False):
        if not self.areas_cache or force:
            self.areas_cache = self._json_request('/areas.json')
        return self.areas_cache

    def reports(self, data, params=None):
        return self._json_request('/reports', method='POST',
                                  data=data, params=params)

    def status(self, rid):
        return self._json_request('/reports/{0}.json'.format(rid))

    def queries(self, rid, qid=None, params=None):
        uri = '/reports/{0}/queries'.format(rid)
        if qid is not None:
            uri += '/' + str(qid)
        uri += '.json'
        return self._json_request(uri, params=params)

    def delete(self, rid):
        return self._json_request('/reports/{0}.json'.format(rid),
                                  method='DELETE')


class Devices(API1Group):
    def __init__(self, *args, **kwargs):
        super(Devices, self).__init__(*args, **kwargs)
        self.device_cache = None        # currently unused
        self.type_cache = None

    def get_all(self, typeid=None, cidr=None, force=False):
        """ Get list of all devices with optional typeid and cidr filtering
        """
        params = {}
        if typeid:
            params['type_id'] = typeid
        if cidr:
            params['cidr'] = cidr
        return self._json_request('', params=params)

    def get_details(self, ipaddr):
        """ Retrieve device instance for a given ip address
        """
        return self._json_request('/{0}.json'.format(str(ipaddr)))

    def get_types(self, force=False):
        """ Get list of unique (type_id, type) pairs for known devices
        """
        if not self.type_cache or force:
            data = self.get_all()
            types = set((x['type_id'], x['type']) for x in data)
            self.type_cache = list(types)
            self.type_cache.sort(key=lambda x: x[0])
        return self.type_cache


class HostGroupTypes(API1Group):
    def __init__(self, *args, **kwargs):
        super(HostGroupTypes, self).__init__(*args, **kwargs)
        self.device_cache = None        # currently unused
        self.type_cache = None

    def get_all(self, favorite=None, offset=None, sortby=None,
                sort=None, type=None, limit=None):
        """ Get a list of all host grouping types
        """
        params = {}
        if favorite:
            params['favorite'] = favorite
        if offset:
            params['offset'] = offset
        if sortby:
            params['sortby'] = sortby
        if sort:
            params['sort'] = sort
        if type:
            params['type'] = type
        if limit:
            params['limit'] = limit
        return self._json_request('', params=params)

    def get_all_groups(self, type_id, offset=None, sortby=None,
                       sort=None, limit=None):
        """ Get a list of all host groups for a given host grouping type
        """
        params = {}
        if offset:
            params['offset'] = offset
        if sortby:
            params['sortby'] = sortby
        if sort:
            params['sort'] = sort
        if limit:
            params['limit'] = limit
        return self._json_request('/{0}/groups'.format(str(type_id)),
                                  params=params)

    def get_config(self, type_id):
        """ Get host grouping type configuration
        """
        params = {}
        return self._json_request('/{0}/config'.format(str(type_id)),
                                  params=params)

    def get(self, type_id):
        """ Get a specific group type element
        """
        params = {}
        return self._json_request('/{0}'.format(str(type_id)), params=params)

    def get_group(self, type_id, group_id):
        """ Get a specific group type element
        """
        params = {}
        return self._json_request('/{0}/groups/{1}'
                                  .format(str(type_id), str(group_id)),
                                  params=params)

    def get_group_members(self, type_id, group_id,
                          offset=None, sort=None,limit=None):
        """ Get a list of hosts in a specified host group
        """
        params = {}
        if offset:
            params['offset'] = offset
        if sort:
            params['sort'] = sort
        if limit:
            params['limit'] = limit
        return self._json_request('/{0}/groups/{1}/members'
                                  .format(str(type_id), str(group_id)),
                                  params=params)

    def create(self, name, desc, favorite, config):
        """ Create a new host grouping type
        """
        params = {}
        data = {}
        data['name'] = name
        data['description'] = desc
        data['favorite'] = favorite
        data['config'] = config
        return self._json_request('', method='POST', data=data, params=params)

    def set_config(self, type_id, config):
        """ Update host grouping type configuration
        """
        params = {}
        return self._json_request('/{0}/config'.format(str(type_id)),
                                  method='PUT', data=config, params=params)

    def set(self, type_id, name, desc, favorite, config):
        """ Update one host grouping type
        """
        params = {}
        data = {}
        data['name'] = name
        data['description'] = desc
        data['favorite'] = favorite
        data['config'] = config
        return self._json_request('/{0}'.format(str(type_id)),
                                  method='PUT', data=data, params=params)

    def delete(self, type_id):
        """ Delete one host grouping type
        """
        params = {}
        return self._json_request('/{0}'.format(str(type_id)),
                                  method='DELETE', params=params)


class Services(API1Group):

    def get_all(self):
        """ Return a list of all services. """
        return self._json_request('')


class Handler(object):
    def __init__(self, profiler):
        self.report = Report('/api/profiler/1.0/reporting', profiler)
        self.devices = Devices('/api/profiler/1.0/devices', profiler)
        self.common = Common('/api/common/1.0', profiler)

        if profiler.supports_version('1.2'):
            self.host_group_types = HostGroupTypes(
                '/api/profiler/1.2/host_group_types', profiler)

        if profiler.supports_version('1.3'):
            self.services = Services(
                '/api/profiler/1.3/services', profiler)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



"""
The NetProfiler package offers a set of interfaces to control and work with
a SteelCentral NetProfiler appliance.
"""
from steelscript.netprofiler.core.netprofiler import *
from steelscript.netprofiler.core.report import *



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.netprofiler.core import NetProfiler
from steelscript.netprofiler.core.hostgroup import HostGroup, HostGroupType
from steelscript.common.service import UserAuth
from steelscript.common.exceptions import RvbdException

import sys
import logging

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

try:
    from testconfig import config
except ImportError:
    if __name__ != '__main__':
        raise
    config = {}

# XXX we try to use unittest.SkipTest() in setUp() below but it
# isn't supported by python 2.6.  this simulates the same thing...
# another 2.6 hack
if 'profilerhost' not in config:
    __test__ = False

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.DEBUG,
                    format="%(asctime)s [%(levelname)-5.5s] %(msg)s")


def create_profiler():
    """ Create NetProfiler instance given configuration data
    """
    if 'profilerhost' not in config:
        raise unittest.SkipTest('no netprofiler hostname provided')
    try:
        username = config['username']
    except KeyError:
        username = 'admin'
    try:
        password = config['password']
    except KeyError:
        password = 'admin'
    auth = UserAuth(username, password)
    return NetProfiler(config['profilerhost'], auth=auth)


class HostGroupTests(unittest.TestCase):
    def setUp(self):
        self.profiler = create_profiler()
        try:
            # TestType4057 is reserved for these tests. 4057 has no significance
            # If we can find that host group for some reason in the netprofiler
            # then we must delete it before starting any of our tests.
            old_host_group_type = HostGroupType.find_by_name(self.profiler,
                                                             'TestType4057')
            old_host_group_type.delete()
        except RvbdException:
            return

    def test_find_by_name(self):
        """ Check that it finds ByLocation as a HostGroupType """
        host_group_type = HostGroupType.find_by_name(self.profiler, "ByLocation")
        self.assertEqual(host_group_type.name, "ByLocation")

    def test_create_save_and_delete(self):
        """ Check that when you create a new group type that it is added """
        host_group_type = HostGroupType.create(self.profiler, "TestType4057")
        host_group_type.save()
        HostGroupType.find_by_name(self.profiler, "TestType4057")
        host_group_type.delete()
        self.assertIsNone(host_group_type.id)

    def test_group_keeptogether_and_append(self):
        """
        Check that you can add host groups and that it responds correctly
        When you append an element to the end with keep_together=True
        """
        host_group_type = HostGroupType.create(self.profiler, "TestType4057")
        # Create a bunch of new host groups for testing
        new_host_groups = []
        for i in range(10):
            new_host_groups.append(HostGroup(host_group_type, 'Test'+str(i)))
            new_host_groups[i].add("10.9{0}.11.0/24".format(i),
                                   keep_together=True, prepend=False)
        # Add our special host group that will be what we focus on
        new_host_groups[9].add("10.10.21.0/24", keep_together=True, prepend=False)
        host_group_type.save()
        # If everything went well, the new host group will be the 2nd element
        self.assertEqual(host_group_type.groups['Test9'].get()[1], "10.10.21.0/24")
        host_group_type.delete()

    def test_group_keeptogether_and_prepend(self):
        """
        Check that you can add host groups and that it responds correctly
        when you prepend an element first on the list that it works
        """
        host_group_type = HostGroupType.create(self.profiler, "TestType4057")
        # Create a bunch of new host groups for testing
        new_host_groups = []
        for i in range(10):
            new_host_groups.append(HostGroup(host_group_type, 'Test'+str(i)))
            new_host_groups[i].add("10.9{0}.11.0/24".format(i),
                                   keep_together=True, prepend=False)
        # Add our special host group that will be what we focus on
        new_host_groups[0].add("10.10.21.0/24", keep_together=True, prepend=True)
        host_group_type.save()
        # If everything went well, the new host group will be the 2nd element
        self.assertEqual(host_group_type.groups['Test0'].get()[0], "10.10.21.0/24")
        host_group_type.delete()

    def test_add_group_to_end_and_start(self):
        """
        Check that add(cidr, keep_together=False, replace=False, prepend=True or
        prepend=False) works as expected
        """
        host_group_type = HostGroupType.create(self.profiler, "TestType4057")
        prepend_group = HostGroup(host_group_type, "PrependGroup")
        prepend_group.add("10.91.11.0/24", keep_together=False, prepend=True)

        append_group = HostGroup(host_group_type, "AppendGroup")
        append_group.add("10.91.11.0/24", keep_together=False, prepend=False)

        self.assertEqual(host_group_type.config[0]['name'], 'PrependGroup')
        self.assertEqual(host_group_type.config[1]['name'], 'AppendGroup')

    def test_replace(self):
        """
        Check that the replace parameter works when adding groups.
        """
        host_group_type = HostGroupType.create(self.profiler, "TestType4057")
        # Create a bunch of new host groups for testing
        new_host_groups = []
        for i in range(10):
            new_host_groups.append(HostGroup(host_group_type, 'Test'+str(i)))
            new_host_groups[i].add("10.9{0}.11.0/24".format(i),
                                   keep_together=True, prepend=False)
        # Add our special host group that will be what we focus on
        new_host_groups[9].add("10.10.21.0/24", keep_together=True,
                               prepend=False, replace=True)
        host_group_type.save()
        # If everything went well, the new host group will be the 1st element
        self.assertEqual(host_group_type.groups['Test9'].get()[0], "10.10.21.0/24")
        self.assertEqual(len(host_group_type.groups['Test9'].get()), 1)
        host_group_type.delete()

    def test_remove_group(self):
        """
        Check that the remove method in HostGroup works properly by adding a few
        cidrs that have the same value, but are in different host groups, then
        removing one of those entries from one host group to make sure the other
        is not effected.
        """
        host_group_type = HostGroupType.create(self.profiler, "TestType4057")
        # Create a bunch of new host groups for testing
        new_host_groups = []
        for i in range(10):
            new_host_groups.append(HostGroup(host_group_type, 'Test'+str(i)))
            new_host_groups[i].add("10.9{0}.11.0/24".format(i),
                                   keep_together=True, prepend=False)
        # Add our special host group that will be what we focus on
        new_host_groups[9].add("10.10.21.0/24", keep_together=True,
                               prepend=False)
        new_host_groups[8].add("10.10.21.0/24", keep_together=True,
                               prepend=False)
        new_host_groups[9].remove("10.10.21.0/24")

        host_group_type.save()
        # If everything went well, the new host group will be the 2nd element
        self.assertEqual(host_group_type.groups['Test9'].get()[0], "10.99.11.0/24")
        self.assertEqual(host_group_type.groups['Test8'].get()[1], "10.10.21.0/24")
        self.assertEqual(len(host_group_type.groups['Test9'].get()), 1)
        host_group_type.delete()

    def test_abusive_input(self):
        """
        Check that after a series of weirdly placed loads, saves, and additions,
        the program handles everything correctly
        """
        host_group_type = HostGroupType.create(self.profiler, "TestType4057")
        self.assertRaises(RvbdException, HostGroup, "Hurtful", host_group_type)
        self.assertRaises(RvbdException, host_group_type.load)
        host_group_type.save()
        host_group_type.delete()
        self.assertRaises(RvbdException, host_group_type.delete)
        host_group_type.create(self.profiler, "AnotherTestType4057", False, "PLE")
        self.assertRaises(RvbdException, host_group_type.load)
        HostGroup(host_group_type, "TestGroup")
        host_group_type.groups['TestGroup'].get()
        host_group_type.config = host_group_type.groups['TestGroup'].get()

    def test_empty_get(self):
        """ Check that you get an empty array when you try to get an empty group
        """
        host_group_type = HostGroupType.create(self.profiler, "TestType4057")
        new_host_group = HostGroup(host_group_type, "EmptyGroup")
        self.assertEqual(new_host_group.get(), [])

    def test_add_cidrs_as_list(self):
        """ Checks if you can add cidrs to a host group as a list
        """
        host_group_type = HostGroupType.create(self.profiler, "TestType4057")
        prepend_group = HostGroup(host_group_type, "PrependGroup")
        prepend_group.add(["10.91.11.0/24", "10.92.11.0/24"],
                          keep_together=False, prepend=True)

        self.assertEqual(host_group_type.config[0]['cidr'], '10.91.11.0/24')
        self.assertEqual(host_group_type.config[1]['cidr'], '10.92.11.0/24')


if __name__ == '__main__':
    # for standalone use take one command-line argument: the netprofiler host
    import sys
    assert len(sys.argv) == 2

    config = {'profilerhost': sys.argv[1]}
    sys.argv = [sys.argv[0]]

    unittest.main()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.netprofiler.core import _constants

"""
This module contains classes to define and collect netprofiler data types
of Column, and Area.
"""
class Container(object):
    pass


class Column(object):
    """A column object represents a single data column in Profiler terms"""
    def __init__(self, cid, key, label, json, baseid=None, ephemeral=False):
        # Numeric column id.  This may be ephemeral -- meaning
        # its a really big number like 100000+.  For a given report
        # all columns in the report must have different ids.
        #
        # Non-ephemeral ids are static, like avg_bytes is always 33
        #
        # Ephemeral ids are dynamic and assigned for a specific
        # report only.  They are used in cases like top-N reporting
        # where there are multiple 'avg_bytes' columns, one for each
        # proto/port (for example).  In this case baseid refers to
        # the base columns type that tells you the general information
        # about the column, and the id will be what you use to request
        # data from the report.
        self.id = cid
        self.key = key
        self.label = label
        self.json = json
        self.iskey = json['category'] != 'data'
        self.baseid = (baseid or cid)
        self.ephemeral = ephemeral

    @classmethod
    def from_json(self, json):
        ephemeral = json['id'] >= _constants.EPHEMERAL_COLID
        strid = json['strid']
        if strid.startswith('ID_'):
            key = strid.lower()[3:]
        else:
            # Current known use cases, this is a number
            # and is equal to str(json['id'])
            key = strid

        return Column(json['id'], key, json['name'],
                      json=json, ephemeral=ephemeral)

    def __eq__(self, other):
        return self.key == other

    def __cmp__(self, other):
        return cmp(self.key, other.key)

    def __hash__(self):
        return hash(tuple(self.json.values()))

    def __repr__(self):
        if self.baseid and self.baseid != self.id:
            msg = ('<steelscript.netprofiler.core._types.Column(id={0} '
                   'key={1} iskey={2} label={3} baseid={4})>')
            return msg.format(self.id, self.key, self.iskey, self.label,
                              self.baseid)
        else:
            msg = ('<steelscript.netprofiler.core._types.Column(id={0} '
                   'key={1} iskey={2} label={3})>')
            return msg.format(self.id, self.key, self.iskey, self.label)



class Area(object):
    def __init__(self, name, key):
        self.name = name
        self.key = key


class AreaContainer(object):
    """Wrapper class for Area objects
    """
    # TODO use actual Area objects in here
    def __init__(self, areas):
        """Initialize with list of key/value pairs
        """
        self._update(areas)

    def _update(self, areas):
        for k, v in areas:
            # TODO tests are looking at keys:values and values:keys
            # do we need bi-directional lookups?
            setattr(self, k, k)
            setattr(self, v, v)


class ColumnContainer(object):
    """Wrapper class for key and value Column classes
    Can be iterated against to get combined results.
    """
    def __init__(self, columns):
        self.key = Container()
        self.value = Container()
        self._map = dict()
        self._update(columns)

    def __getitem__(self, key):
        return self._map[key]

    def __iter__(self):
        """Iterates over keys and values to provide combined Column results.
        """
        for c in self.keys:
            yield c
        for c in self.values:
            yield c

    def __contains__(self, key_or_id):
        return key_or_id in self._map

    def _update(self, columns):
        """Take list of Column objects and apply their keys and ids as attributes.
        """
        for c in columns:
            self._map[c.key] = c
            self._map[c.id] = c
            if c.iskey:
                setattr(self.key, c.key, c)
            else:
                setattr(self.value, c.key, c)

    @property
    def keys(self):
        """Return the collection of 'key' Column objects.
        """
        try:
            return self.key.__dict__.values()
        except AttributeError:
            return None

    @property
    def values(self):
        """Return the collection of 'value' Column objects.
        """
        try:
            return self.value.__dict__.values()
        except AttributeError:
            return None



# !/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

from steelscript.netprofiler.core import *
from steelscript.netprofiler.core.app import NetProfilerApp
from steelscript.netprofiler.core.filters import TimeFilter, TrafficFilter

import pprint
import optparse


class TrafficToHostGroupApp(NetProfilerApp):
    def add_options(self, parser):
        super(TrafficToHostGroupApp, self).add_standard_options()
        group = optparse.OptionGroup(self.parser, "Host Group Parameters")
        group.add_option(
            '-g', '--group', action='store', default=None,
            help='[REQUIRED] type and group to add gathered data to,'
                 ' separated by a :, ex: "ByLocation:SanFrancisco"')
        group.add_option(
            '-t', '--trafficexpr', action='store', default=None,
            help='[REQUIRED] traffic expression, ex: "host 10/8"')
        group.add_option(
            '-f', '--timefilter', action='store', default="last 5m",
            help='time filter for query, ex: "last 5m"')
        group.add_option(
            '-a', '--append', action='store_true', dest='append', default=False,
            help='append rather than replace host group config')
        self.parser.add_option_group(group)

    def validate_args(self):
        super(TrafficToHostGroupApp, self).validate_args()
        if self.options.group is None:
            self.parser.error("Missing required option: GROUP")
        if self.options.trafficexpr is None:
            self.parser.error("Missing required option: TRAFFICEXPR")

    def main(self):
        # Create and run a traffic summary report of all hosts in use
        # and then take that data and send it to a specified host group
        report = TrafficSummaryReport(self.netprofiler)

        # Run the report
        report.run(
            groupby=self.netprofiler.groupbys.host,
            columns=[self.netprofiler.columns.key.host_ip,
                     self.netprofiler.columns.key.group_name],
            sort_col=self.netprofiler.columns.key.group_name,
            timefilter=TimeFilter.parse_range(self.options.timefilter),
            trafficexpr=TrafficFilter(self.options.trafficexpr)
        )

        # Store the report's data
        data = report.get_data()
        # Grab the type_name and group_name from options.group
        (type_name, group_name) = self.options.group.split(':', 1)
        # Create an array to store the new config data
        new_config_entries = []

        # Using data from the report, put it in config-readable format.
        for i in range(len(data)):
            new_config_entries.append({'cidr': data[i][0] + '/32',
                                       'name': group_name})
        # Make sure that if there were no entries returned,
        # we don't overwrite the old data
        if len(new_config_entries) == 0:
            print('ERROR: Report returned zero hosts for supplied parameters')
            return

        # Get the ID of the host type specified by name
        host_types = self.netprofiler.api.host_group_types.get_all()
        target_type_id = -1
        for i, host_type in enumerate(host_types):
            if type_name == host_type['name']:
                target_type_id = host_type['id']
                break
        # If target_type_id is still -1, then we didn't find that host
        if target_type_id == -1:
            print('ERROR: Host Group Type: "' + type_name + '" was not found.')
            return

        # Get the current config from the target host group
        config = self.netprofiler.api.host_group_types.get_config(target_type_id)
        old_config_size = len(config)
        # If the append flag is not true,
        # remove all entries in config matching group_name
        if self.options.append is False:
            config = filter(lambda a: a['name'] != group_name, config)

        config.extend(new_config_entries)
        new_config_size = len(config)
        self.netprofiler.api.host_group_types.set_config(target_type_id, config)
        print("Successfully updated type: " + type_name +
              ", group: " + group_name)
        print("The old config had " + str(old_config_size) +
              " elements. It now has " + str(new_config_size) + " elements.\n")


TrafficToHostGroupApp().run()



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import numpy
import paramiko
import optparse
import matplotlib.pyplot as plt

from steelscript.netprofiler.core.app import NetProfilerApp
from steelscript.netprofiler.core import *
from steelscript.netprofiler.core.filters import TimeFilter, TrafficFilter


class PercentileApp(NetProfilerApp):
    def add_options(self, parser):
        super(PercentileApp, self).add_options(parser)

        group = optparse.OptionGroup(parser, 'data processing')
        group.add_option(
            '-b', '--buckettime', default=5, type='int',
            help="Number of minutes data should be bucketed in (default: 5)")
        group.add_option(
            '-e', '--percentile', default=95, type='int',
            help="Percentile to display data at (default: 95)")

        group = optparse.OptionGroup(parser, 'time frame')
        group.add_option('-t', '--timefilter', default="last 5 m",
            help=("Time filter expression in the form \"last x [{m|h|d|w}]\" "
                  "or \"10-30-2012 10:30am to 10-31-2012 10:02am.\" Default: "
                  "last 5 m"))
        parser.add_option_group(group)

        group = optparse.OptionGroup(parser, 'traffic filter')
        group.add_option(
            '-r', '--trafficfilter', default="host 10/8",
            help=("Traffic filter expression to use (e.g., \"host 10/8\"). "
                  "When querying hostgroups, you can also enter just the host "
                  "group type (e.g., \"hostgroup ByLocation\")."))
        parser.add_option_group(group)

        group = optparse.OptionGroup(parser, 'output')
        group.add_option('--median', action='store_true', default=False,
                         help="Show the median from the data set")
        group.add_option('--min', action='store_true', default=False,
                         help="Show the minimum from the data set")
        group.add_option('--max', action='store_true', default=False,
                         help="Show the maximum from the data set")
        group.add_option(
            '--clean', action='store_true', default=False,
            help="Show only the name and percentile value")

        group.add_option(
            '-o', '--rawdata', action='store_true', default=False,
            help="Additionally display the raw data used to determine the "
                 "percentile values")
        parser.add_option_group(group)

        group = optparse.OptionGroup(parser, 'ssh options')
        group.add_option("--sshusername", help="Username for SSH")
        group.add_option("--sshpassword", help="Password for SSH")
        parser.add_option_group(group)



        group = optparse.OptionGroup(parser, 'list groups')
        group.add_option(
            '--listinterfacegroups', action='store_true', default=False,
            help=("List all available interface groups. Separate hierarchies "
                  "with slashes, e.g. ByRegion/North_America. (Requires "
                  "--sshusername and --sshpassword to be defined.)"))
        group.add_option('--listhostgroups', action='store_true', default=False,
                         help=("List all available host groups, organized by "
                               "group type"))
        parser.add_option_group(group)

        group = optparse.OptionGroup(parser, 'graph options')
        group.add_option('--graph',
                         help="Save a time series graph in the provided path.")
        group.add_option('--percentileline', action='store_true', default=True,
                         help="Display a line at the chosen percentile")
        group.add_option('--startzero', action='store_true', default=False,
                         help="Start y axis at zero")
        parser.add_option_group(group)

    def validate_args(self):
        super(PercentileApp, self).validate_args()

        if not 1 <= self.options.percentile <= 100:
            self.parser.error("Percentile must be between 1 and 100")

        if (self.options.listinterfacegroups and
            (self.options.sshpassword is None or self.options.sshusername is None)):
            self.parser.error("When using --listinterfacegroups, --sshusername "
                              "and --sshpassword must be provided.")

    def gen_graph(self, rawdata, bucketed_data, percentile):
        percentile_val = numpy.percentile(bucketed_data, percentile)
        endx = len(bucketed_data) * self.options.buckettime

        if self.options.percentileline:
            plt.axhline(percentile_val, color='r',
                        label="{0}% = {1:,.2f} bytes".format(percentile, percentile_val))

        plt.xlim(xmax=endx)
        plt.xlabel("Time (minutes)")
        plt.xticks(rotation="vertical")

        plt.ylabel("Bytes")

        plt.title("NetProfiler Traffic Data: " + self.options.trafficfilter)
        plt.legend()

        plt.plot(xrange(0, endx, self.options.buckettime), bucketed_data,
                 label=self.options.trafficfilter)

        # If --startzero is set, start y axis at zero (naturally). Otherwise
        # keep the min value set by the autoscale feature of plot()
        ymin = 0 if self.options.startzero else plt.ylim()[0]
        # Add a little vertical space for legend
        ymax = int(max(bucketed_data) * 1.2)
        plt.ylim(ymin, ymax)

        plt.savefig(self.options.graph)

    def bucket_data(self, rawdata, buckettime):
        bucketed_data = []
        bucketvals = []
        for i, avg_bytes in enumerate(rawdata):
            bucketvals.append(avg_bytes)
            if i != 0 and (i % buckettime == 0 or i == (len(rawdata) - 1)):
                # Current bucket is full -- save data and start a new empty bucket.
                bucketed_data.append(numpy.average(bucketvals))
                bucketvals = []

        return bucketed_data

    def list_interface_groups(self, host, sshusername, sshpassword):
        # There appears to be no REST API for getting interface groups, so run
        # a command over SSH and pass along the results.
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(host, username=sshusername, password=sshpassword)
        stdout = ssh.exec_command("/usr/mazu/bin/mazu-interfacegroups -l")[1]
        print stdout.read()
        ssh.close()

    def list_host_groups(self, profiler):
        for grouptype in profiler.api.host_group_types.get_all():
            print "Group type:", grouptype['name']

            for group in profiler.api.host_group_types.get_all_groups(grouptype['id']):
                print " " + group['name']

    def report_item(self, profiler, timefilter, trafficfilter,
                    buckettime, percentile):
        """ Calculate and display the percentile report for one item
            (host, interface, hostgroup, etc.)"""
        exptype, _, item_id = trafficfilter.filter.partition(" ")

        centricity = ('hos' if exptype == 'hos' else 'int')

        with TrafficOverallTimeSeriesReport(profiler) as report:
            report.run([profiler.columns.value.avg_bytes],
                       timefilter=timefilter,
                       trafficexpr=trafficfilter,
                       centricity=centricity,
                       resolution='1min')
            report.wait_for_complete()
            data = report.get_data()

        rawdata = [avg_bytes for (avg_bytes, ) in data]
        bucketed_data = self.bucket_data(rawdata, buckettime)

        if self.options.clean:
            print "{} {:,.2f}".format(self.options.trafficfilter,
                                      numpy.percentile(bucketed_data, percentile))
        else:
            if self.options.rawdata:
                print "Raw data points:", ", ".join(str(val) for val in rawdata)

            print "Average bytes at percentile {0}: {1:,.2f}".format(
                  self.options.percentile,
                  numpy.percentile(bucketed_data, percentile))

            if self.options.median:
                print ("Median average bytes: "
                       "{0:,.2f}").format(numpy.percentile(bucketed_data, 50))

            if self.options.max or self.options.min:
                print
                if self.options.max:
                    print "Max average bytes: {0:,.2f}".format(max(bucketed_data))
                if self.options.min:
                    print "Min average bytes: {0:,.2f}".format(min(bucketed_data))

        if self.options.graph:
            self.gen_graph(rawdata, bucketed_data, percentile)
            if not self.options.clean:
                print
                print "Graph saved to", self.options.graph

    def main(self):
        if self.options.listinterfacegroups:
            self.list_interface_groups(self.options.host, self.options.sshusername,
                                       self.options.sshpassword)
            return

        if self.options.listhostgroups:
            self.list_host_groups(NetProfiler(self.options.host, auth=self.auth))
            return

        try:
            timefilter = TimeFilter.parse_range(self.options.timefilter)
        except ValueError:
            print "Could not parse time filter expression."
            return

        profiler = NetProfiler(self.options.host, auth=self.auth)

        if not self.options.clean:
            print "Reporting on the period: {}".format(self.options.timefilter)
            print "Using the traffic filter: {}".format(self.options.trafficfilter)
            print "Calculating data at percentile {}".format(self.options.percentile)
            print ("Averaging based on buckets of {} "
                   "minutes").format(self.options.buckettime)
            if self.options.graph:
                print "Saving a graph to {}".format(self.options.graph)
            print

        trafficfilter = TrafficFilter(self.options.trafficfilter)

        self.report_item(profiler, timefilter, trafficfilter,
                         self.options.buckettime, self.options.percentile)

if __name__ == "__main__":
    PercentileApp().run()



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



from steelscript.netprofiler.core.app import NetProfilerApp
from steelscript.netprofiler.core.filters import TimeFilter, TrafficFilter
from steelscript.netprofiler.core.report import (IdentityReport,
                                                 TrafficOverallTimeSeriesReport,
                                                 TrafficSummaryReport)
from steelscript.common.datautils import Formatter
from steelscript.common.timeutils import string_to_datetime

import sys
import imp
import datetime
import optparse
import itertools


AGGREGATION = {'total': lambda x: sum(x),
               'avg'  : lambda x: sum(x) / len(x),
               'peak' : lambda x: max(x),
               'min'  : lambda x: min(x)}

# Columns for Time Series Report
TCOLUMNS = [
    ('time', AGGREGATION['min']),
    ('total_bytes', AGGREGATION['total']),
    ('avg_bytes', AGGREGATION['avg']),
    ('network_rtt', AGGREGATION['peak']),
    ('response_time', AGGREGATION['peak']),
    ('server_delay', AGGREGATION['peak']),
    ('avg_conns_rsts', AGGREGATION['avg']),
    ('avg_pkts_rtx', AGGREGATION['avg']),
    ('avg_rsec_jitter', AGGREGATION['avg']),
    ('avg_vqual_mos', AGGREGATION['min']),
]

# Columns for Traffic Summary Report
SCOLUMNS = [
    ('total_bytes', AGGREGATION['total']),
    ('avg_bytes', AGGREGATION['avg']),
    ('network_rtt', AGGREGATION['peak']),
    ('response_time', AGGREGATION['peak']),
    ('server_delay', AGGREGATION['peak']),
    ('avg_conns_rsts', AGGREGATION['avg']),
    ('avg_pkts_rtx', AGGREGATION['avg']),
    ('avg_rsec_jitter', AGGREGATION['avg']),
    ('avg_vqual_mos', AGGREGATION['min']),
]


def format_time(value):
    """Convenience function to translate timestamp to ISO format"""
    t = datetime.datetime.fromtimestamp(value)
    return t.isoformat(' ')


class IdentityApp(NetProfilerApp):

    def add_options(self, parser):
        super(IdentityApp, self).add_options(parser)
        group = optparse.OptionGroup(parser, 'Identity Report Options')
        group.add_option('-n', '--identity-name', dest='identity_name',
                         help='Login name to use for search')
        group.add_option('-T', '--traffic-filter', dest='trafficexpr', default=None,
                         help='Traffic filter to narrow down IP address '
                              'search space')

        group.add_option('--time0', dest='time0', default=None,
                         help='Start time for report')
        group.add_option('--time1', dest='time1', default=None,
                         help='End time for report')
        group.add_option('-r', '--timerange', dest='timerange', default=None,
                         help='Optional time range in place of t0 and t1')
        group.add_option('-b', '--backsearch', dest='backsearch', default='24',
                         help='Hours to look backwards to find possible identity match '
                              'defaults to the maximum of "24" hours')

        group.add_option('--resolution', default='auto',
                         help='Time resolution to use for report queries, '
                              'defaults to "auto", may be one of the following: '
                              '("1min", "15min", "hour", "6hour", "day")')
        group.add_option('--timeseries-report', dest='timeseries_report', default=False,
                         action='store_true',
                         help='Run time series traffic reports for hosts during found '
                              'login times.')
        group.add_option('--aggregate', dest='aggregate', default=False,
                         action='store_true',
                         help='Set to group timeseries data into single row per timeperiod.')

        group.add_option('--summary-report', dest='summary_report', default=False,
                         action='store_true',
                         help='Run summary reports for hosts during found login times.')

        group.add_option('--groupby-application', dest='groupby_application', default=False,
                         action='store_true',
                         help='Run summary reports for hosts during found login times.')
        group.add_option('--groupby-interface', dest='groupby_interface', default=False,
                         action='store_true',
                         help='Run summary reports for hosts during found login times.')

        group.add_option('--csv', dest='csv', default=False, action='store_true',
                         help='Print results in CSV table.')
        group.add_option('--tsv', dest='tsv', default=False, action='store_true',
                         help='Print results in TSV (tab-separated-values) table.')

        group.add_option('--testfile', dest='testfile', default=None,
                         help='Optional test file with identity events to use in place of '
                              'actual netprofiler queries.')
        group.add_option('--usecache', dest='usecache', default=False, action='store_true',
                         help='Use internal cache to help with large traffic query sets')

        parser.add_option_group(group)

    def validate_args(self):
        """ Check that either both t0 and t1 are used or time range
        """
        super(IdentityApp, self).validate_args()

        if self.options.timerange and (self.options.time0 or
                                       self.options.time1):
            self.parser.error('timerange and t0/t1 are mutually exclusive, '
                              'choose only one.')

        elif (not self.options.timerange and
              not self.options.time0 and
              not self.options.time1):
            self.parser.error('A timerange must be chosen.')

        elif not self.options.identity_name:
            self.parser.error('An identity_name must be chosen.')

        elif int(self.options.backsearch) > 24:
            self.parser.error('Time for back search cannot exceed "24" hours.')

        elif self.options.timeseries_report and self.options.summary_report:
            self.parser.error('Only one report type may be selected at a time.')

    def identity_report(self, timefilter=None, trafficexpr=None, testfile=None):
        """ Run IdentityReport and return data
        """
        identity = self.options.identity_name

        if not testfile:
            # run report against all users
            print 'Running IdentityReport ...'
            report = IdentityReport(self.netprofiler)
            report.run(timefilter=timefilter, trafficexpr=trafficexpr)
            print 'Report complete, gathering data ...'
            data = report.get_data()

            if identity not in (x[1] for x in data):
                print 'Running report farther back to find identity ...'
                delta = datetime.timedelta(hours=int(self.options.backsearch))
                timefilter.start = timefilter.start - delta
                report.run(timefilter=timefilter, trafficexpr=trafficexpr)
                data = report.get_data()

            if not data:
                print "Empty data results."

            legend = report.get_legend()
            report.delete()
        else:
            print 'Reading from testfile %s ...' % testfile
            try:
                f, path, desc = imp.find_module(testfile)
                test = imp.load_module(testfile, f, path, desc)
                data = test.data
                legend = self.netprofiler.get_columns(test.legend)
            except ImportError:
                print 'Error importing test file %s' % testfile
                print 'Ensure it is in the PYTHONPATH, and contains a valid data object.'
                sys.exit(1)
            finally:
                f.close()

        return legend, data

    def traffic_report(self, host, timefilter, report_type):
        """ Generate average statistics for host given `timefilter` time period

            `report_type` is one of ('timeseries', 'summary')
        """
        print 'Running %s report for %s over %s/%s' % (report_type,
                                                       host,
                                                       timefilter.start,
                                                       timefilter.end)

        texpr = TrafficFilter('host %s' % host)

        if report_type == 'timeseries':
            columns = [c[0] for c in TCOLUMNS]
            report = TrafficOverallTimeSeriesReport(self.netprofiler)
            report.run(columns,
                       timefilter=timefilter,
                       trafficexpr=texpr,
                       resolution=self.options.resolution)
        elif report_type == 'summary':
            columns = [c[0] for c in SCOLUMNS]
            report = TrafficSummaryReport(self.netprofiler)

            if self.options.groupby_application:
                columns.insert(0, 'app_name')
                groupby = 'app'
            elif self.options.groupby_interface:
                columns.insert(0, 'interface_alias')
                columns.insert(0, 'interface')
                groupby = 'ifc'
            else:
                groupby = 'hos'

            report.run(groupby,
                       columns,
                       timefilter=timefilter,
                       trafficexpr=texpr,
                       resolution=self.options.resolution)
        else:
            raise RuntimeError('unknown report type: %s' % report_type)

        print 'Report complete, gathering data ...'
        data = report.get_data()
        if not data:
            print "Empty data results."
        elif len(data) == 10000:
            print 'WARNING: data size exceeds max length of 10000 rows'
        legend = report.get_legend()
        report.delete()

        return legend, data

    def analyze_login_data(self, all_data, legend_columns, single_ip=True):
        """ Identify periods user logged into each host

            `single-ip` indicates that a user may only have one IP at a time.
                Logins indicating a different IP address will mean previous
                IP address has been released.
        """
        logins = {}  # scratch lookup table
        current_ip = None  # last ip of user
        activity = []  # sequence of events

        identity = self.options.identity_name

        legend_keys = [c.key for c in legend_columns]

        # all_data contains a time-ordered list of logins for all users
        # the sort order is latest to earliest, so we want to parse this in the
        # reverse order
        for login in all_data[::-1]:
            login = dict(zip(legend_keys, login))

            time = login['time']
            host = login['host_dns'].strip('|')
            user = login['username']

            if single_ip:
                if current_ip and (host == current_ip or user == identity):
                    # ip changes to new user or user gets assigned new ip
                    start = logins.pop(current_ip)
                    duration = time - start
                    activity.append((current_ip, start, time, duration))
                    current_ip = None

                if user == identity:
                    logins[host] = time
                    current_ip = host
            else:
                if host in logins:
                    # new login to existing host
                    start = logins.pop(host)
                    duration = time - start
                    activity.append((host, start, time, duration))
                    current_ip = None

                if user == identity:
                    logins[host] = time
                    current_ip = host

        activity.sort(key=lambda x: x[1])
        legend = ['Host IP', 'Login Time', 'Logout Time', 'Duration']
        return legend, activity

    def generate_traffic(self, activity, legend_keys, report_type):
        """ Generate traffic data during the time the user was logged-in.
        """
        cache = {}
        combined_activity = []
        for event in activity:
            # handle dns names in host along with IP address
            host = event[0].split('|', 1)[0]

            timefilter = TimeFilter(string_to_datetime(event[1]),
                                    string_to_datetime(event[2]))
            # if event occurs in less than a minute, add extra minute to report
            while len(timefilter.profiler_minutes()) == 1:
                timefilter.end += datetime.timedelta(minutes=1)

            # normalize times to minute increments
            mins = timefilter.profiler_minutes()
            tf = TimeFilter(mins[0], mins[-1])

            if self.options.usecache and report_type == 'timeseries':
                # only consider a hit when whole time period is covered
                minutes = tf.profiler_minutes(astimestamp=True)

                if host in cache and all(t in cache[host] for t in minutes):
                    data = [cache[host][t] for t in minutes]
                else:
                    legend, data = self.traffic_report(host, tf, report_type)
                    # store results in cache by host->times->data
                    cache.setdefault(host, {}).update((int(x[0]), x) for x in data)
            else:
                legend, data = self.traffic_report(host, tf, report_type)

            if data:
                if self.options.aggregate and report_type == 'timeseries':
                    # generate running averages over data samples received
                    # first convert empty strings to zeros, then run averages
                    columns = map(lambda c: [0 if x == '' else x for x in c],
                                                                itertools.izip(*data))
                    aggmap = [x[1] for x in TCOLUMNS]
                    aggregates = [aggmap[i](x) for i, x in enumerate(columns)]
                    combined_activity.append(list(event) + aggregates)
                elif report_type == 'timeseries' or report_type == 'summary':
                    # create entry for each element in report
                    for row in data:
                        r = ['--' if x == '' else x for x in row]
                        combined_activity.append(list(event) + r)
                else:
                    raise RuntimeError('unknown report type: %s' % report_type)

            else:
                # populate result with blanks
                combined_activity.append(list(event) + ['--'] * len(legend))

        traffic_legend = [c.key for c in legend]

        legend = legend_keys + traffic_legend
        return legend, combined_activity

    def main(self):
        """ Setup query and run report with default column set
        """
        if self.options.timerange:
            timefilter = TimeFilter.parse_range(self.options.timerange)
        else:
            timefilter = TimeFilter(self.options.time0, self.options.time1)

        if self.options.trafficexpr:
            trafficexpr = TrafficFilter(self.options.trafficexpr)
        else:
            trafficexpr = None

        legend_columns, all_data = self.identity_report(timefilter=timefilter,
                                                        trafficexpr=trafficexpr,
                                                        testfile=self.options.testfile)

        legend, activity = self.analyze_login_data(all_data, legend_columns)

        if activity and self.options.timeseries_report:
            headers, tbl_data = self.generate_traffic(activity, legend, 'timeseries')
        elif activity and self.options.summary_report:
            headers, tbl_data = self.generate_traffic(activity, legend, 'summary')
        else:
            headers = ('Host IP', 'Login Time', 'Logout Time', 'Duration')
            tbl_data = [(x[0], format_time(x[1]), format_time(x[2]), x[3])
                                                                for x in activity]

        if self.options.csv:
            Formatter.print_csv(tbl_data, headers)
        elif self.options.tsv:
            Formatter.print_csv(tbl_data, headers, delim='\t')
        else:
            Formatter.print_table(tbl_data, headers)


if __name__ == '__main__':
    IdentityApp().run()



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.netprofiler.core.app import NetProfilerApp
from steelscript.netprofiler.core.filters import TimeFilter, TrafficFilter
from steelscript.netprofiler.core.report import TrafficFlowListReport
from steelscript.common.datautils import Formatter

import optparse


class NetProfilerReport(NetProfilerApp):

    def add_options(self, parser):
        super(NetProfilerReport, self).add_options(parser)
        group = optparse.OptionGroup(parser, "Traffic Flow Options")
        group.add_option('--time0', dest='time0', default=None,
                         help='Start time for report')
        group.add_option('--time1', dest='time1', default=None,
                         help='End time for report')
        group.add_option('-r', '--timerange', dest='timerange', default=None,
                         help='Optional time range in place of t0 and t1')
        group.add_option('-e', '--traffic-expression', dest='trafficexpr',
                         default=None, help='Traffic Expression to query on')
        parser.add_option_group(group)

    def validate_args(self):
        """ Check that either both t0 and t1 are used or timerange
        """
        super(NetProfilerReport, self).validate_args()

        if self.options.timerange and (self.options.time0 or
                                       self.options.time1):
            self.parser.error('timerange and t0/t1 are mutually exclusive, '
                              'choose only one.')

    def main(self):
        """ Setup query and run report with default column set
        """
        if self.options.timerange:
            timefilter = TimeFilter.parse_range(self.options.timerange)
        else:
            timefilter = TimeFilter(self.options.time0, self.options.time1)
        trafficexpr = TrafficFilter(self.options.trafficexpr)

        columns = [self.netprofiler.columns.key.srv_host_ip,
                   self.netprofiler.columns.key.app_info,
                   self.netprofiler.columns.key.start_time,
                   self.netprofiler.columns.key.end_time,
                   self.netprofiler.columns.value.s2c_total_bytes,
                   self.netprofiler.columns.value.s2c_total_pkts,
                   self.netprofiler.columns.value.response_time,
                   self.netprofiler.columns.value.server_delay]

        report = TrafficFlowListReport(self.netprofiler)
        report.run(columns, timefilter=timefilter, trafficexpr=trafficexpr)
        data = report.get_data()
        report.delete()

        headers = [c.key for c in columns]

        Formatter.print_table(data, headers)


if __name__ == '__main__':
    NetProfilerReport().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



from steelscript.netprofiler.core import NetProfiler
from steelscript.netprofiler.core.filters import TimeFilter, TrafficFilter
from steelscript.common.service import UserAuth
from steelscript.common.exceptions import RvbdException
from steelscript.netprofiler.core.report import (WANSummaryReport, WANTimeSeriesReport, TrafficSummaryReport,
                                  TrafficOverallTimeSeriesReport, TrafficFlowListReport,
                                  IdentityReport)

import unittest
import logging
import datetime

try:
    from testconfig import config
except ImportError:
    if __name__ != '__main__':
        raise
    config = {}

# XXX we try to use unittest.SkipTest() in setUp() below but it
# isn't supported by python 2.6.  this simulates the same thing...
# another 2.6 hack
if 'profilerhost' not in config:
    __test__ = False

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.DEBUG,
                    format="%(asctime)s [%(levelname)-5.5s] %(msg)s")


def create_profiler():
    """ Create NetProfiler instance given configuration data
    """
    if 'profilerhost' not in config:
        raise unittest.SkipTest('no netprofiler hostname provided')
    try:
        username = config['username']
    except KeyError:
        username = 'admin'
    try:
        password = config['password']
    except KeyError:
        password = 'admin'
    auth = UserAuth(username, password)
    return NetProfiler(config['profilerhost'], auth=auth)


class ProfilerTests(unittest.TestCase):
    def setUp(self):
        self.profiler = create_profiler()
        y = datetime.datetime.now() - datetime.timedelta(days=1)
        yesterday_at_4 = datetime.datetime(y.year, y.month, y.day, hour=16, minute=0, microsecond=1)
        yesterday_at_5 = datetime.datetime(y.year, y.month, y.day, hour=17, minute=0, microsecond=1)
        self.yesterday = TimeFilter(yesterday_at_4, yesterday_at_5)

    def test_groupby_structure(self):
        self.assertTrue('port' in self.profiler.groupbys)
        self.assertEqual(self.profiler.groupbys.port, 'por')

    def test_columns_structure(self):
        self.assertEqual(self.profiler.columns.key.host_ip, 'host_ip')
        dir(self.profiler.columns.key)
        self.assertEqual(self.profiler.columns.key.host_ip.id, 5)

    def test_search_columns(self):
        all_columns = self.profiler.search_columns()
        groupby_hos = self.profiler.search_columns(groupbys=['hos'])
        centricities_and_groupby = self.profiler.search_columns(centricities=['hos'],
                                                                groupbys=['tim'])
        assert len(all_columns) > 0
        assert len(groupby_hos) > 0
        assert len(centricities_and_groupby) > 0
        assert len(all_columns) != len(groupby_hos)
        assert len(all_columns) != len(centricities_and_groupby)
        assert len(groupby_hos) != len(centricities_and_groupby)
        logger.debug("Retrieved columns for groupby 'hos': %s" % groupby_hos)

    def test_get_columns(self):
        names = ['time', 'host_ip', 'network_rtt']
        cols = self.profiler.get_columns(names)
        self.assertEqual(len(cols), 3)
        self.assertEqual(cols[0].id, 98)
        self.assertEqual(cols[1].id, 5)
        self.assertEqual(cols[2].id, 280)
        self.assertRaises(RvbdException, self.profiler.get_columns, ['noexist'])
        # test __cmp__ method on Column object
        self.assertTrue(self.profiler.columns.key.time >
                        self.profiler.columns.key.host_ip)

    def test_get_columns_by_ids(self):
        ids = [98, 5, 280]
        cols = self.profiler.get_columns_by_ids(ids)
        self.assertTrue(len(cols) == 3)
        keys = [c.key for c in cols]
        self.assertTrue('time' in keys)
        self.assertTrue('host_ip' in keys)
        self.assertTrue('network_rtt' in keys)

    def test_logout(self):
        pass

    def test_timefilter(self):
        tfilter = TimeFilter.parse_range('9:01:36 to 10:04:39')

        testtime = tfilter.start.replace(minute=33, second=59)
        self.assertTrue(tfilter.compare_time(testtime))
        testtime = tfilter.end.replace(minute=44)
        self.assertFalse(tfilter.compare_time(testtime))

        minutes = tfilter.profiler_minutes()
        self.assertEqual(len(minutes), 64)
        minutes = tfilter.profiler_minutes(astimestamp=True)
        self.assertEqual(len(minutes), 64)
        minutes = tfilter.profiler_minutes(aslocal=True)
        self.assertEqual(len(minutes), 64)

        tfilter = TimeFilter.parse_range('9:01:36 to 9:02:33')
        minutes = tfilter.profiler_minutes()
        self.assertEqual(len(minutes), 1)

    def test_traffic_summary_report(self):
        groupby = self.profiler.groupbys.host
        columns = [self.profiler.columns.key.host_ip,
                   self.profiler.columns.value.avg_bytes,
                   self.profiler.columns.value.avg_pkts]
        sort_col = self.profiler.columns.value.avg_bytes
        timerange = TimeFilter.parse_range("last 1 h")
        trafficexpr = TrafficFilter("host 10/8")

        with TrafficSummaryReport(self.profiler) as rep:
            rep.run(groupby, columns,
                    sort_col, timerange,
                    trafficexpr)
            legend = rep.get_legend()
            self.assertEqual(len(legend), 3)
            legend = rep.get_legend(columns=[self.profiler.columns.key.host_ip,
                                             self.profiler.columns.value.avg_bytes])
            self.assertEqual(len(legend), 2)
            self.assertEqual(legend[0].key, 'host_ip')
            self.assertEqual(legend[1].key, 'avg_bytes')

            data = rep.get_data()
            if data:
                self.assertEqual(len(data[0]), 3)

            #check that data is refetched from cache
            data = rep.get_data()

            data = rep.get_data(columns=[self.profiler.columns.key.host_ip,
                                         self.profiler.columns.value.avg_bytes])
            if data:
                self.assertEqual(len(data[0]), 2)

    def test_traffic_overall_time_series_report(self):

        columns = [self.profiler.columns.key.time,
                   self.profiler.columns.value.avg_bytes,
                   self.profiler.columns.value.avg_pkts]

        timerange = TimeFilter.parse_range("last 1 h")
        trafficexpr = TrafficFilter("host 10/8")
        resolution = "15min"
        report = TrafficOverallTimeSeriesReport(self.profiler)
        report.run(columns, timerange, trafficexpr, resolution=resolution)

        legend = report.get_legend()
        keys = [c.key for c in legend]

        data = report.get_data()

        for item in data:
            d = dict(zip(keys, item))
            # resolution assumes 15-minute responses
            self.assertTrue(timerange.compare_time(d['time'], resolution=15*60))
        report.delete()

    def test_traffic_flow_list_report(self):
        columns = [self.profiler.columns.key.srv_host_ip,
                   self.profiler.columns.key.app_info,
                   self.profiler.columns.key.start_time,
                   self.profiler.columns.key.end_time,
                   self.profiler.columns.value.s2c_total_bytes,
                   self.profiler.columns.value.s2c_total_pkts,
                   self.profiler.columns.value.response_time,
                   self.profiler.columns.value.server_delay]
        timerange = TimeFilter.parse_range("last 1 h")
        trafficexpr = TrafficFilter("host 10/8")

        with TrafficFlowListReport(self.profiler) as report:
            report.run(columns, timefilter=timerange, trafficexpr=trafficexpr)

            legend = report.get_legend()
            keys = [c.key for c in legend]
            self.assertTrue('app_info' in keys)

            data = report.get_data()
            if data:
                self.assertEqual(len(data[0]), 8)

    def test_identity_report(self):
        timerange = TimeFilter.parse_range('last 30 m')

        with IdentityReport(self.profiler) as report:
            report.run(timefilter=timerange)
            legend = report.get_legend()
            data = report.get_data()
            keys = [c.key for c in legend]
            self.assertTrue('time' in keys)
            self.assertTrue('username' in keys)
            if data:
                self.assertEqual(len(data[0]), 9)

    def test_unsupported_column(self):
        groupby = self.profiler.groupbys.port

        # host_ip shouldn't be included as part of 'port' groupby
        columns = [self.profiler.columns.key.host_ip,
                   self.profiler.columns.value.avg_bytes,
                   self.profiler.columns.value.avg_pkts]
        sort_col = self.profiler.columns.value.avg_bytes

        timerange = TimeFilter.parse_range("last 1 h")
        trafficexpr = TrafficFilter("host 10/8")

        report = TrafficSummaryReport(self.profiler)
        kwds = dict(groupby=groupby,
                    columns=columns,
                    sort_col=sort_col,
                    timefilter=timerange,
                    trafficexpr=trafficexpr)
        self.assertRaises(RvbdException, report.run, None, kwds)

    def test_resolution(self):
        groupby = self.profiler.groupbys.host
        columns = [self.profiler.columns.key.host_ip,
                   self.profiler.columns.value.avg_bytes,
                   self.profiler.columns.value.avg_pkts]
        sort_col = self.profiler.columns.value.avg_bytes
        trafficexpr = TrafficFilter("host 10/8")
        resolutions = [["1min", "last 5 min"],
                       ["15min", "last 1 hour"],
                       ["hour", "last 4 hours"],
                       ["6hour", "last 1 day"],
                       ["day", "last 1 week"],
                       ["3600", "last 4 hours"],  # hour resolution
                       ["60", "last 5 min"],      # minute resolution
                       #["week", "last 4 weeks"],
                       #["month", "last 12 months"],
                       #Commented values blow up with a
                       #E       RvbdHTTPException: 400 Unknown time resolution.
                       ]
        for (resolution, duration) in resolutions:
            timerange = TimeFilter.parse_range(duration)
            with TrafficSummaryReport(self.profiler) as rep:
                rep.run(groupby, columns,
                        sort_col, timerange,
                        trafficexpr, resolution=resolution)

    def test_area(self):
        self.assertEqual(self.profiler.areas.wan, 'wan')
        self.assertEqual(self.profiler.areas.lan, 'lan')
        self.assertEqual(self.profiler.areas.vxlan_tenant, 'vxlan_tenant')
        self.assertEqual(self.profiler.areas.vxlan_tunnel, 'vxlan_tunnel')
        self.assertEqual(self.profiler.areas.vxlan, 'vxlan')

        # do we really need this bi-directional support?
        self.assertEqual(self.profiler.areas.wan, 'wan')
        self.assertEqual(self.profiler.areas.lan, 'lan')
        self.assertEqual(self.profiler.areas.tnt, 'tnt')
        self.assertEqual(self.profiler.areas.prd, 'prd')
        self.assertEqual(self.profiler.areas.vxl, 'vxl')

    def test_report_with_area(self):
        groupby = self.profiler.groupbys.host
        columns = [self.profiler.columns.key.host_ip,
                   self.profiler.columns.value.avg_bytes,
                   self.profiler.columns.value.avg_pkts]
        sort_col = self.profiler.columns.value.avg_bytes
        timerange = TimeFilter.parse_range("last 1 h")
        trafficexpr = TrafficFilter("host 10/8")
        area = self.profiler.areas.vxlan_tenant
        with TrafficSummaryReport(self.profiler) as rep:
            rep.run(groupby, columns,
                    sort_col, timerange,
                    trafficexpr, area=area)

    def test_wan_time_series_report(self):
        # WAN reports depend on pandas module, so skip if we don't have it
        try:
            import pandas
        except ImportError:
            return

        # we don't have a way to find WAN interfaces yet, so this test is
        # dependent on a netprofiler running the demo traffic program
        # if we can't find the 'Austin' interface, just end test
        ip_address = None
        devices = self.profiler.api.devices.get_all()
        for d in devices:
            if 'sh-austin' in d['name'].lower():
                ip_address = d['ipaddr']
        if not ip_address:
            return

        columns = ['time',
                   'avg_bytes',
                   'total_bytes']
        self.groupby = None

        with WANTimeSeriesReport(self.profiler) as report:
            lan_address, wan_address = report.get_interfaces(ip_address)
            self.assertTrue(lan_address)
            self.assertTrue(wan_address)

            report.run(lan_address, wan_address, 'inbound', columns=columns,
                       timefilter=self.yesterday, resolution='1min')
            inbound = report.get_data(as_list=False)
            self.assertEqual(inbound.shape, (60,4))
            self.assertTrue(all(inbound.LAN_avg_bytes > inbound.WAN_avg_bytes))

            report.run(lan_address, wan_address, 'outbound', columns=columns,
                       timefilter=self.yesterday, resolution='1min')
            outbound = report.get_data(as_list=False)
            self.assertEqual(outbound.shape, (60,4))
            self.assertTrue(all(outbound.LAN_avg_bytes > outbound.WAN_avg_bytes))

    def test_wan_time_summary_report(self):
        # WAN reports depend on pandas module, so skip if we don't have it
        try:
            import pandas
        except ImportError:
            return

        # we don't have a way to find WAN interfaces yet, so this test is
        # dependent on a netprofiler running the demo traffic program
        # if we can't find the 'Austin' interface, just end test
        ip_address = None
        devices = self.profiler.api.devices.get_all()
        for d in devices:
            if 'sh-austin' in d['name'].lower():
                ip_address = d['ipaddr']
        if not ip_address:
            return

        columns = ['device',
                   'avg_bytes',
                   'total_bytes']
        self.groupby = 'dev'

        with WANSummaryReport(self.profiler) as report:
            lan_address, wan_address = report.get_interfaces(ip_address)
            self.assertTrue(lan_address)
            self.assertTrue(wan_address)

            report.run(lan_address, wan_address, 'inbound', columns=columns,
                       groupby=self.groupby,
                       timefilter=self.yesterday)
            inbound = report.get_data(as_list=False)

            self.assertEqual(inbound.shape, (1,4))
            self.assertTrue(all(inbound.LAN_avg_bytes > inbound.WAN_avg_bytes))

            report.run(lan_address, wan_address, 'outbound', columns=columns,
                       groupby=self.groupby,
                       timefilter=self.yesterday)
            outbound = report.get_data(as_list=False)
            self.assertEqual(outbound.shape, (1,4))
            self.assertTrue(all(outbound.LAN_avg_bytes > outbound.WAN_avg_bytes))


class ProfilerDevicesTests(unittest.TestCase):
    def setUp(self):
        self.profiler = create_profiler()
        # common header fields for device lists
        self.headernames = ['name', 'type_id', 'ipaddr', 'version', 'type', 'id']

    def test_typelist(self):
        """Check that get_types returns list of two-tuples"""
        types = self.profiler.api.devices.get_types()
        self.assertEqual(len(types[0]), 2)

    def test_getdevices(self):
        """Verify list of dicts for devices returned on get_all query"""
        devices = self.profiler.api.devices.get_all()
        dev0 = devices[0]
        self.assertTrue(isinstance(dev0, dict))
        for h in self.headernames:
            self.assertTrue(h in dev0.keys())

    def test_getdevices_by_type(self):
        """Verify list of dicts for devices returned on get_all filtered query"""
        # pull type id from first item in list, then confirm its included
        # when filtering on its type id
        devices = self.profiler.api.devices.get_all()
        dev0 = devices[0]
        typeid = dev0['type_id']
        filtered = self.profiler.api.devices.get_all(typeid=typeid)
        self.assertTrue(dev0 in filtered)

    def test_getdevices_by_cidr(self):
        """Verify list of dicts for devices returned on get_all filtered query"""
        # pull ip from first result, and check its in the result set when
        # querying for that
        devices = self.profiler.api.devices.get_all()
        dev0 = devices[0]
        ipaddr = dev0['ipaddr']
        root = ipaddr.split('.')[0]
        query = '%s.0.0.0/8' % root
        filtered = self.profiler.api.devices.get_all(cidr=query)
        self.assertTrue(dev0 in filtered)
        empty = self.profiler.api.devices.get_all(cidr='10.0.0.0/32')
        self.assertFalse(empty)

    def test_getdetails(self):
        """Verify query by ip address returns single result"""
        devices = self.profiler.api.devices.get_all()
        ipaddr = devices[0]['ipaddr']
        dev = self.profiler.api.devices.get_details(ipaddr)
        self.assertTrue(isinstance(dev, dict))
        for h in self.headernames:
            self.assertTrue(h in dev.keys())


if __name__ == '__main__':
    # for standalone use take one command-line argument: the netprofiler host
    import sys
    assert len(sys.argv) == 2

    config = {'profilerhost': sys.argv[1]}
    sys.argv = [sys.argv[0]]

    unittest.main()



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


"""
Simple example demonstrating ability for multi-stage user defined policy.

Requires installation of `pysnmp` python library to support sending snmp traps

example call signature:
python alerter.py tm08-1.lab.nbttech.com -u admin -p admin --expr1 'app WEB' --trigger1 '> 100' \
    --column1 'avg_bytes' --expr2 'app VoIP' --trigger2 '> 6100' --column2 'avg_bytes'

"""

import sys
import datetime

from steelscript.netprofiler.core.app import NetProfilerApp
from steelscript.netprofiler.core.filters import TrafficFilter
from steelscript.netprofiler.core.report import TrafficSummaryReport
from steelscript.common import timeutils

from pysnmp.entity.rfc3413.oneliner import ntforg
from pysnmp.proto import rfc1902

import re
import time
import optparse


def safe_lambda(trigger):
    """ Return a reasonably safe lambda function for the given trigger
    """
    regex = re.compile('^[ <>=]+[0-9.]+[ ]*$')
    trigger = trigger.strip().strip('"').strip("'")
    if re.match(regex, trigger):
        return lambda x: eval('%s %s' % (x, trigger))
    return None


class AlerterApp(NetProfilerApp):

    def add_options(self, parser):
        super(AlerterApp, self).add_options(parser)
        group = optparse.OptionGroup(parser, "Trigger Options")
        group.add_option('--expr1',
                         help="Initial traffic expression to trigger on")
        group.add_option('--trigger1',
                         help="First Trigger (e.g. '> 1000')")
        group.add_option('--column1',
                         help="Column to apply trigger against (e.g. 'avg_bytes')")
        group.add_option('--expr2',
                         help="(Optional) Secondary traffic expression to trigger on")
        group.add_option('--trigger2',
                         help="(Optional) Second Trigger (e.g. '< 10')")
        group.add_option('--column2',
                         help="(Optional) Column to apply trigger against (e.g. 'network_rtt')")
        group.add_option('--refresh',
                         help="Refresh interval for queries in Seconds (default: 5)",
                         default=5)
        parser.add_option_group(group)

        group = optparse.OptionGroup(parser, "TRAP Options")
        group.add_option('--eoid',
                         help="Enterprise OID (defaults to SteelCentral NetProfiler ID: '1.3.6.1.4.1.7054.70.0.' )",
                         default='1.3.6.1.4.1.7054.70.0.')
        group.add_option('--trapid', help="Trap code indicator (default: 99)",
                         default='99')
        group.add_option('--community',
                         help="Community name (default: 'public')",
                         default='public')
        group.add_option('--manager_ip',
                         help="Destination server for Traps (default: '127.0.0.1')",
                         default='127.0.0.1')
        group.add_option('--trap-description',
                         help="Description to use within Trap (default: 'SteelScript SDK Trap Alert')",
                         default='FlyScript SDK Trap Alert')
        group.add_option('--severity', help="Severity of Trap (default: '70')",
                         default=70)
        group.add_option('--trap-url',
                         help="URL to include in Trap Message (default: 'http://localhost')",
                         default='http://localhost')
        group.add_option('--alert-level',
                         help="Alert level of Trap (1=Low, 2=Medium, 3=High) (default: 2)",
                         default=2)
        parser.add_option_group(group)

    def validate_args(self):
        super(AlerterApp, self).validate_args()

        if not self.options.trigger1:
            self.parser.error('At least trigger1 must be specified.')

    def send_trap(self):
        """ Send a SNMP trap with id `trapid` to the IP address `manager`
        """
        oid = self.options.eoid  # cascade enterprise Object ID
        trapid = self.options.trapid  # base string for trap indicators

        community = self.options.community
        manager_ip = self.options.manager_ip

        severity = self.options.severity
        description = self.trap_description
        url = self.options.trap_url
        alert_level = self.options.alert_level
        now = timeutils.datetime_to_seconds(datetime.datetime.now())

        trapname = '.'.join([oid, trapid])

        ntf = ntforg.NotificationOriginator()

        err = ntf.sendNotification(
            ntforg.CommunityData(community),
            ntforg.UdpTransportTarget((manager_ip, 162)),
            'trap',
            trapname,
            ('1.3.6.1.2.1.1.3.0', rfc1902.Integer(0)),                        # Uptime
            ('1.3.6.1.4.1.7054.71.2.1.0', rfc1902.Integer(severity)),         # Severity
            ('1.3.6.1.4.1.7054.71.2.3.0', rfc1902.OctetString(description)),
            ('1.3.6.1.4.1.7054.71.2.4.0', rfc1902.Integer(0)),                # Event ID
            ('1.3.6.1.4.1.7054.71.2.5.0', rfc1902.OctetString(url)),
            ('1.3.6.1.4.1.7054.71.2.7.0', rfc1902.Integer(alert_level)),      # Alert Level
            ('1.3.6.1.4.1.7054.71.2.8.0', rfc1902.Integer(now)),              # Start Time
            ('1.3.6.1.4.1.7054.71.2.16.0', rfc1902.Integer(0)),               # Source Count
            ('1.3.6.1.4.1.7054.71.2.18.0', rfc1902.Integer(0)),               # Destination Count
            ('1.3.6.1.4.1.7054.71.2.20.0', rfc1902.Integer(0)),               # Protocol Count
            ('1.3.6.1.4.1.7054.71.2.22.0', rfc1902.Integer(0)),               # Port Count
        )

    def run_query(self, report, column, trafficexpr, trigger):
        report.run(columns=column, groupby='hos', trafficexpr=trafficexpr)
        data = report.get_data()
        if data:
            for row in data:
                for item in row:
                    if item and trigger(item):
                        return item
        return None

    def update_description(self, result, result2):
        """ Creates string to highlight what threshold/trigger was crossed
        """
        desc = ''
        if result:
            desc = 'Trigger1: %s %s' % (result, self.options.trigger1)
            if result2:
                desc = '%s --> Trigger2: %s %s' % (desc, result2, self.options.trigger2)
        if desc:
            self.trap_description = '%s: %s' % (self.options.trap_description, desc)
        else:
            self.trap_description = self.options.trap_description

    def main(self):
        self.tfilter1 = TrafficFilter(self.options.expr1)
        self.report1 = TrafficSummaryReport(self.netprofiler)
        self.columns_report1 = self.netprofiler.get_columns([self.options.column1])
        self.trigger1 = safe_lambda(self.options.trigger1)

        if self.options.trigger2 and self.options.column2 and self.options.expr2:
            self.tfilter2 = TrafficFilter(self.options.expr2)
            self.report2 = TrafficSummaryReport(self.netprofiler)
            self.columns_report2 = self.netprofiler.get_columns([self.options.column2])
            self.trigger2 = safe_lambda(self.options.trigger2)
        else:
            self.trigger2 = None

        try:
            while 1:
                print 'Running report 1 ...'
                alert_flag = False
                result = None
                result2 = None

                result = self.run_query(self.report1,
                                        self.columns_report1,
                                        self.tfilter1,
                                        self.trigger1)
                if result:
                    if self.trigger2:
                        print 'Trigger 1 passed, running report 2 ...'
                        result2 = self.run_query(self.report2,
                                                 self.columns_report2,
                                                 self.tfilter2,
                                                 self.trigger2)
                        if result2:
                            print 'Trigger 2 passed ...'
                            alert_flag = True
                    else:
                        print 'Trigger 1 passed ...'
                        alert_flag = True
                if alert_flag:
                    print 'ALERT ALERT!'
                    self.update_description(result, result2)
                    self.send_trap()

                time.sleep(self.options.refresh)

        except KeyboardInterrupt:
            print 'Exiting ...'
            sys.exit(0)


if __name__ == '__main__':
    AlerterApp().run()



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.netprofiler.core.app import NetProfilerApp
from steelscript.netprofiler.core.report import TrafficSummaryReport
from steelscript.netprofiler.core.filters import TimeFilter, TrafficFilter
from steelscript.common.datautils import Formatter

import optparse


class TrafficSummaryApp(NetProfilerApp):

    def add_options(self, parser):
        super(TrafficSummaryApp, self).add_options(parser)
        group = optparse.OptionGroup(parser, "Report Parameters")
        group.add_option('--centricity', dest='centricity', default='host',
                         help='"host" vs "interface" centricity (default "host")')
        group.add_option('--groupby', dest='groupby', default='host',
                         help='Groupby for report data (default "host")')
        group.add_option('--columns', dest='columns',
                         help='Comma-separated list of column names and/or '
                              'ID numbers, required')
        parser.add_option_group(group)

        group = optparse.OptionGroup(parser, "Filter Options")
        group.add_option('--timefilter', dest='timefilter', default='last 1 hour',
                         help='Time range to analyze (defaults to "last 1 hour") '
                              'other valid formats are: "4/21/13 4:00 to 4/21/13 5:00" '
                              'or "16:00:00 to 21:00:04.546"')

        group.add_option('--trafficexpr', dest='trafficexpr', default=None,
                         help='Traffic Expression to apply to report (default None)')
        parser.add_option_group(group)

        group = optparse.OptionGroup(parser, "Output options")
        group.add_option('--sort', dest='sortby', default=None,
                         help='Column name to sort by (defaults to None)')
        group.add_option('--csv', dest='as_csv', default=False, action='store_true',
                         help='Return values in CSV format instead of tabular')
        parser.add_option_group(group)

    def validate_args(self):
        """ Ensure columns are included
        """
        super(TrafficSummaryApp, self).validate_args()

        if self.options.centricity == 'host':
            self.centricity = 'hos'
        elif self.options.centricity == 'interface':
            self.centricity = 'int'
        elif self.options.centricity not in ['hos', 'int']:
            self.parser.error('Centricity option must be either "int" or "hos".')
        else:
            self.centricity = self.options.centricity

        if not self.options.columns:
            self.parser.error('Comma-separated list of columns is required.')

    def print_data(self, data, header):
        if self.options.as_csv:
            Formatter.print_csv(data, header)
        else:
            Formatter.print_table(data, header)

    def main(self):
        # groupby validation should be part of validate_args, but there
        # is no NetProfiler initialized at that part of the initialization
        try:
            self.groupby = self.netprofiler.groupbys[self.options.groupby]
        except KeyError:
            if self.options.groupby not in self.netprofiler.groupbys.values():
                self.parser.error('Invalid groupby chosen.')
            else:
                self.groupby = self.options.groupby

        self.timefilter = TimeFilter.parse_range(self.options.timefilter)
        if self.options.trafficexpr:
            self.trafficexpr = TrafficFilter(self.options.trafficexpr)
        else:
            self.trafficexpr = None

        with TrafficSummaryReport(self.netprofiler) as report:
            report.run(columns=self.options.columns.split(','),
                       groupby=self.groupby,
                       sort_col=self.options.sortby,
                       timefilter=self.timefilter,
                       trafficexpr=self.trafficexpr,
                       centricity=self.centricity,
                       resolution='auto')
            data = report.get_data()
            legend = [c.label for c in report.get_legend()]

        self.print_data(data, legend)


if __name__ == '__main__':
    TrafficSummaryApp().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.appfwk.apps.report.models import Report
import steelscript.appfwk.apps.report.modules.yui3 as yui3

from steelscript.netprofiler.appfwk.datasources.netprofiler import \
    NetProfilerServiceByLocTable

#
# NetProfiler report
#

report = Report.create("NetProfiler Services", position=10)

report.add_section()

# Define a Overall TimeSeries showing Avg Bytes/s
p = NetProfilerServiceByLocTable.create('services-by-loc')

report.add_widget(yui3.TableWidget, p, "Services by location", width=6)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.appfwk.apps.report.models import Report
import steelscript.appfwk.apps.report.modules.yui3 as yui3

from steelscript.netprofiler.appfwk.datasources.netprofiler_devices import NetProfilerDeviceTable

report = Report.create("NetProfiler Device List", position=10)

report.add_section()

#
# Device Table

p = NetProfilerDeviceTable.create('devtable')
p.add_column('ipaddr', 'Device IP', iskey=True, datatype="string")
p.add_column('name', 'Device Name', datatype="string", sortasc=True)
p.add_column('type', 'Flow Type', datatype="string")
p.add_column('version', 'Flow Version', datatype="string")

report.add_widget(yui3.TableWidget, p, "Device List", height=300, width=12)



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import optparse

from steelscript.netprofiler.core.app import NetProfilerApp
from steelscript.netprofiler.core.filters import TimeFilter
from steelscript.netprofiler.core.report import \
    TrafficSummaryReport, TrafficTimeSeriesReport
from steelscript.common.datautils import Formatter


class TopPortsTime(NetProfilerApp):

    def add_options(self, parser):
        super(TopPortsTime, self).add_options(parser)

        group = optparse.OptionGroup(parser, "Report Options")
        group.add_option(
            '--timefilter',
            default='last 15 min',
            help=('Time range to analyze (defaults to "last 15 min") '
                  'other valid formats are: "4/21/13 4:00 to 4/21/13 5:00" '
                  'or "16:00:00 to 21:00:04.546"'))

        group.add_option(
            '-N',
            default=10,
            help=('Top N to report on'))

        parser.add_option_group(group)

    def main(self):
        netprof = self.netprofiler

        timefilter = TimeFilter.parse_range(self.options.timefilter)

        # Create and run a traffic summary report of all server ports in use
        report = TrafficSummaryReport(netprof)

        # Run the report
        report.run(
            groupby=netprof.groupbys.port,
            columns=[netprof.columns.key.protoport,
                     netprof.columns.key.protocol,
                     netprof.columns.key.port,
                     netprof.columns.value.avg_bytes],
            sort_col=netprof.columns.value.avg_bytes,
            timefilter=timefilter)

        # Retrieve and print data
        ports_data = report.get_data()[:int(self.options.N)]
        report.delete()

        # Now create a new report using the ports_data
        report = TrafficTimeSeriesReport(netprof)

        # The format the query_columns for 'ports' is:
        #    'ports' = [{'name': 'tcp/80'},
        #               {'name': 'tcp/443'},
        #               {'name': 'icmp/0'}]
        # For most protocols, this works just fine from the report data,
        # but for icmp the result from data is 'icmp/0/0' -- where the two
        # zeros are type and code.  This doesn't work for input to
        # netprofiler, it expects type and code to be smushed into a single
        # 16-bit number (type << 8 | code).
        query_columns = []
        for (protoport, protocol, port, avgbytes) in ports_data:
            if protoport.startswith('icmp'):
                protoport = 'icmp/%s' % (port)

            query_columns.append({'name': protoport})

        # Run the report
        report.run(columns=[netprof.columns.key.time,
                            netprof.columns.value.avg_bytes],
                   resolution='1 min',
                   query_columns_groupby='ports',
                   query_columns=query_columns,
                   timefilter=timefilter)

        # Get the data!
        data = report.get_data()
        Formatter.print_table(
            data, padding=1,
            headers=(['time'] + [q['name'] for q in query_columns]))


if __name__ == "__main__":
    TopPortsTime().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



'''
Created on Jul 20, 2012

@author: vincenzo.ampolo@riverbed.com
'''

class ProfilerException(Exception): pass

class ProfilerHTTPException(ProfilerException):
    def __init__(self, *args, **kwargs):
        ProfilerException.__init__(self, *args, **kwargs)
        
class InvalidGroupbyException(ProfilerException):
    def __init__(self, *args, **kwargs):
        ProfilerException.__init__(self, *args, **kwargs)
        
class InvalidColumnException(ProfilerException):
    def __init__(self, *args, **kwargs):
        super(InvalidColumnException, self).__init__(*args, **kwargs)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from django.conf import settings

from steelscript.appfwk.apps.alerting.senders import ConsoleSender
from steelscript.appfwk.apps.alerting.datastructures import Results

import logging
logger = logging.getLogger(__name__)


class LocalConsoleSender(ConsoleSender):
    def send(self, alert):
        s = getattr(settings, 'FOO', 34)
        print "LocalConsoleSender: Works! %s - %s (%s)" % (alert.level,
                                                           alert.message,
                                                           s)


def simple_trigger(df, context, params):
    """Find any values in column that exceed a given value.

    Required params:
    `column`: specify column to evaluate
    `value`:  threshold value, rows which exceed this value will
        be returned from the trigger
    """
    return Results().add_result((df[params['column']] > params['value']).any(),
                                severity=5)


def local_spike(df, context, params):
    """Find conditions where local spikes occur.

    Can operate against timeseries or other data.  Optional params:

    `column`: specify column to calculate against, defaults to first non-time
        column.
    `std`:    number of standard deviations to consider as a spike,
        defaults to 2
    """
    column = params.get('column', None)
    if column is None:
        if 'time' in df:
            column = df.drop('time', axis=1).columns[0]
        else:
            column = df.columns[0]

    std = params.get('std', 2)

    # extract the column as a Series
    s = df[column]
    delta = s.std() * std

    # create a boolean index where the values exceed the delta threshold
    idx = abs(s - s.mean()) > delta

    # then use this to create a new dataframe
    results = df[idx]

    # as an example, dynamically determine severity based on size of results
    severity = len(results) * 10 if len(results) < 10 else 99

    return Results().add_results(results, severity=severity)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.appfwk.apps.report.models import Report
import steelscript.appfwk.apps.report.modules.yui3 as yui3

from steelscript.netprofiler.appfwk.datasources.netprofiler import \
    NetProfilerTimeSeriesTable, NetProfilerGroupbyTable

# Trigger related imports
from steelscript.appfwk.apps.alerting.models import create_trigger
import netprofiler_triggers

#
# Description
#
description = """
<div style="width:500px">
<p>This example report demonstrates usage of the alerting functionality.

<p>The report gets defined as usual, with tables for datasources and widgets
for display of the data.  Separately, we define a set of trigger functions
that will analyze the results of the table before it gets displayed.  These
triggers are attachd to the table using the ``create_trigger`` function.

<p>If the trigger evaluates to True, the results can be forwarded to a variety
of ``Destinations`` using a ``Sender`` class.  Multiple ``Destinations``
can be added to a single trigger.
</div>
"""

report = Report.create("NetProfiler - Alert Example",
                       description=description,
                       position=10)
report.add_section()

# Define a Overall TimeSeries showing Avg Bytes/s
p = NetProfilerTimeSeriesTable.create('ts-overall', duration=60, resolution="1min")

p.add_column('time', 'Time', datatype='time', iskey=True)
p.add_column('avg_bytes', 'Avg Bytes/s', units='B/s')

report.add_widget(yui3.TimeSeriesWidget, p, "Overall Traffic", width=6)

# Add a trigger to evaluate if traffic exceeds a certain threshold
a = create_trigger(source=p,
                   trigger_func=netprofiler_triggers.local_spike,
                   params={'column': 'avg_bytes',
                           'std': 2})
a.add_destination(
    'LoggingSender',
    options={'level': 'info'},
    template='Logging Local Spike: time - {time}, value - {avg_bytes}'
)

# Define a Table for Response Times
p = NetProfilerGroupbyTable.create('location-resptime', groupby='host_group', duration=60)

p.add_column('group_name', 'Group Name', iskey=True)
p.add_column('response_time', 'Response Time', units='ms', sortdesc=True)

report.add_widget(yui3.BarWidget, p, "Locations by Response Time")

# Add a trigger to evaluate if response time exceeds a certain threshold
a = create_trigger(source=p,
                   trigger_func=netprofiler_triggers.simple_trigger,
                   params={'column': 'response_time',
                           'value': 0.3})
a.add_destination(
    'LoggingSender',
    options={'level': 'info'},
    template='Logging Response Time Exceeded Threshold'
)



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.netprofiler.core import *
from steelscript.netprofiler.core.app import NetProfilerApp
from steelscript.netprofiler.core.filters import TimeFilter, TrafficFilter

import pprint


class TopPortsApp(NetProfilerApp):

    def main(self):
        # Create and run a traffic summary report of all server ports in use
        # by hosts in 10/8
        report = TrafficSummaryReport(self.netprofiler)

        # Run the report
        report.run(
            groupby=self.netprofiler.groupbys.port,
            columns=[self.netprofiler.columns.key.protoport,
                     self.netprofiler.columns.key.protoport_name,
                     self.netprofiler.columns.value.avg_bytes,
                     self.netprofiler.columns.value.network_rtt],
            sort_col=self.netprofiler.columns.value.avg_bytes,
            timefilter=TimeFilter.parse_range("last 15 m"),
            trafficexpr=TrafficFilter("host 10/8")
        )

        # Retrieve and print data
        data = report.get_data()
        printer = pprint.PrettyPrinter(2)
        printer.pprint(data[:20])

TopPortsApp().run()



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.netprofiler.core.app import NetProfilerApp
from steelscript.netprofiler.core.services import ServiceLocationReport
from steelscript.netprofiler.core.filters import TimeFilter

import pprint

app = NetProfilerApp()
app.run()

# Create and run a traffic summary report of all server ports in use
# by hosts in 10/8
report = ServiceLocationReport(app.netprofiler)

# Run the report
report.run(
    timefilter=TimeFilter.parse_range("last 1h")
    )

# Retrieve and print data
data = report.get_data()
printer = pprint.PrettyPrinter(2)
printer.pprint(data[:20])



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

import csv
import sys
import string
import optparse
from collections import defaultdict

from steelscript.netprofiler.core.app import NetProfilerApp
from steelscript.netprofiler.core.hostgroup import HostGroupType, HostGroup
from steelscript.commands.steel import prompt_yn
from steelscript.common.exceptions import RvbdException


# This script will take a file with subnets and SiteNames
# and create a HostGroupType on the target NetProfiler.
# If the HostGroupType already exists, it will be deleted,
# before creating a new one with the same name.
#
# See the EXAMPLE text below for the format of the input
# file.  Note that multiple SiteNames with different
# IP address spaces can be included.


EXAMPLE = """
subnet SiteName
10.143.58.64/26 CZ-Prague-HG
10.194.32.0/23 MX-SantaFe-HG
10.170.55.0/24 KR-Seoul-HG
10.234.9.0/24 ID-Surabaya-HG
10.143.58.63/23 CZ-Prague-HG
"""


class HostGroupImport(NetProfilerApp):

    def add_options(self, parser):
        super(HostGroupImport, self).add_options(parser)
        group = optparse.OptionGroup(parser, "HostGroup Options")
        group.add_option('--hostgroup', action='store',
                         help='Name of hostgroup to overwrite')
        group.add_option('-i', '--input-file', action='store',
                         help='File path to hostgroup file')
        parser.add_option_group(group)

    def validate_args(self):
        """Ensure all arguments are present."""
        super(HostGroupImport, self).validate_args()

        if not self.options.input_file:
            self.parser.error('Host group file is required, specify with '
                              '"-i" or "--input-file"')

        if not self.options.hostgroup:
            self.parser.error('Hostgroup name is required, specify with '
                              '"--hostgroup"')

    def validate(self, name):
        valid = set(string.letters + string.digits + '.-_')
        return all(c in valid for c in name)

    def import_file(self):
        """Process the input file and load into dict."""
        groups = defaultdict(list)

        with open(self.options.input_file, 'rb') as f:
            dialect = csv.Sniffer().sniff(f.read(1024))
            f.seek(0)
            reader = csv.reader(f, dialect)
            header = reader.next()
            if header != ['subnet', 'SiteName']:
                print 'Invalid file format'
                print 'Ensure file has correct header.'
                print 'example file:'
                print EXAMPLE

            for i, row in enumerate(reader):
                cidr, group = row
                if not self.validate(group):
                    print 'Invalid group name on line %d: %s' % (i+2, group)
                    sys.exit()
                groups[group].append(cidr)

        return groups

    def update_hostgroups(self, groups):
        """Replace existing HostGroupType with contents of groups dict."""
        # First find any existing HostGroupType
        try:
            hgtype = HostGroupType.find_by_name(self.netprofiler,
                                                self.options.hostgroup)
            hgtype.config = []
            hgtype.groups = {}
            print ('Existing HostGroupType "%s" found.'
                   % self.options.hostgroup)
        except RvbdException:
            print 'No existing HostGroupType found, creating a new one.'

            hgtype = HostGroupType.create(self.netprofiler,
                                          self.options.hostgroup)

        # Add new values
        for group, cidrs in groups.iteritems():
            hg = HostGroup(hgtype, group)
            hg.add(cidrs)

        # Save to NetProfiler
        hgtype.save()
        print ('HostGroupType "%s" configuration saved.'
               % self.options.hostgroup)

    def main(self):
        """Confirm overwrite then update hostgroups."""

        confirm = ('The contents of hostgroup %s will be overwritten '
                   'by the file %s, are you sure?'
                   % (self.options.hostgroup, self.options.input_file))
        if not prompt_yn(confirm):
            print 'Okay, aborting.'
            sys.exit()

        groups = self.import_file()
        self.update_hostgroups(groups)
        print 'Successfully updated %s on %s' % (self.options.hostgroup,
                                                 self.netprofiler.host)


if __name__ == '__main__':
    HostGroupImport().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

"""
The Host Group module provides an interface for manipulating host group types
and their host groups and hosts.
"""

import re

from steelscript.netprofiler.core import _constants
from steelscript.netprofiler.core.report import SingleQueryReport

service_tree_key_ctxt_re = re.compile(
    '^(?P<col_id>[0-9]+):(?P<elem_id>[0-9:]+)\|(?P<elem_name>[^[]+)')

COL_ID_LOCATION = 691
COL_ID_METRIC_CAT = 692

service_health_ctxt_location_re = re.compile(
    '(?P<health>[0-9]*)\*?'
    '\[service_location_id=(?P<location_id>[0-9:]+)'
    '\[svc_location_id')


service_health_ctxt_location_service_re = re.compile(
    '(?P<health>[0-9]*)\*?'
    '\[service_location_id=(?P<location_id>[0-9:]+)'
    ',service_id=(?P<service_id>[0-9]+)'
    '\[svc_location_id')

service_health_ctxt_metric_cat_re = re.compile(
    '([0-9]*)\*?\[service_location_id=([0-9:]+)'
    ',metric_cat_id=([0-9])+'
    '\[svc_metric_cat_id')

service_health_ctxt_metric_cat_service_re = re.compile(
    '([0-9]*)\*?\[service_location_id=([0-9:]+)'
    ',metric_cat_id=([0-9])+'
    ',service_id=([0-9]+)'
    '\[svc_metric_cat_id')


class Service(object):
    SVC_NOT_AVAILABLE = 0
    SVC_DISABLED = 1
    SVC_INIT = 2
    SVC_NORMAL = 3
    SVC_LOW = 4
    SVC_MED = 5       # supposedly ignored
    SVC_HIGH = 6
    SVC_NODATA = 7


class ServiceLocationReport(SingleQueryReport):

    COLUMNS = ['idx', 'parent_id',
               'tree_key_ctxt', 'tree_key_id', 'tree_key_type',
               'health_ctxt']

    def run(self, **kwargs):
        # Key kwargs: timefilter, sync
        super(ServiceLocationReport, self).run(
            groupby='slm',
            realm='msq',
            columns=self.COLUMNS,
            **kwargs)

    def _get_parsed_data(self):
        raw = super(ServiceLocationReport, self).get_data()

        # Raw data comes back with the following columns:
        #   idx                - row index
        #   parent_id          - parent index (blank for location summary)
        #   tree_key_ctxt      - the context for this row, either a location
        #                        or a metric category
        #   tree_key_id        - the location id or metric cat id for this row
        #   tree_key_type      - the type of row, location or metric cat
        #   health_ctxt        - the actual health, plus full context
        #   [svc_health_ctxt]  - health_ctxt for each service

        pos = {'svc_health_ctxt': []}
        for i, l in enumerate(self.get_legend()):
            if l.id < _constants.EPHEMERAL_COLID:
                pos[l.key] = i
            else:
                pos['svc_health_ctxt'].append([i, l.json['name']])

        rows = []
        for rawrow in raw:
            row = {}
            row['id'] = rawrow[pos['idx']]
            try:
                row['parent_id'] = int(rawrow[pos['parent_id']])
            except ValueError:
                row['parent_id'] = None

            m = service_tree_key_ctxt_re.match(rawrow[pos['tree_key_ctxt']])
            if not m:
                raise ValueError('Failed to parse tree_key_ctxt for row: %s' %
                                 (rawrow[pos['tree_key_ctxt']]))

            if int(m.group('col_id')) != COL_ID_LOCATION:
                # For now, skip sub-levels that are metric categories
                continue

            (byloc, location) = m.group('elem_name').split(':')
            row['location'] = location

            m = service_health_ctxt_location_re.match(
                rawrow[pos['health_ctxt']])
            if not m:
                raise ValueError('Failed to parse overall health_ctxt: %s' %
                                 (rawrow[pos['health_ctxt']]))

            try:
                h = int(m.group('health'))
            except ValueError:
                h = Service.SVC_NOT_AVAILABLE

            row['overall'] = h

            row['services'] = {}
            for idx, svc_name in pos['svc_health_ctxt']:
                m = service_health_ctxt_location_service_re.match(rawrow[idx])
                if not m:
                    raise ValueError(
                        'Failed to parse service %s (%s) health_ctx: %s' %
                        (svc_name, i, rawrow[idx]))

                try:
                    h = int(m.group('health'))
                except ValueError:
                    h = Service.SVC_NOT_AVAILABLE
                row['services'][svc_name] = h

            rows.append(row)

        return rows

    def get_data(self):
        parsed_rows = self._get_parsed_data()

        rows = []
        for prow in parsed_rows:
            row = {}
            row['location'] = prow['location']
            for svc, health in prow['services'].iteritems():
                row[svc] = health
            rows.append(row)

        return rows



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.netprofiler.core.app import NetProfilerApp
from steelscript.common.datautils import Formatter

import optparse

import logging
logger = logging.getLogger(__name__)


class DeviceReport(NetProfilerApp):

    def add_options(self, parser):
        super(DeviceReport, self).add_options(parser)
        group = optparse.OptionGroup(parser, "Device List Options")
        group.add_option('-t', '--typeid', dest='typeid', default=None,
                         help='filter by type id integer')
        group.add_option('-c', '--cidr', dest='cidr', default=None,
                         help='filter by cidr subnets')
        group.add_option('-i', '--ipaddr', dest='ipaddr', default=None,
                         help='return details for specific ip address')
        group.add_option('--typelist', action='store_true', dest='typelist',
                         default=False, help='return type information')
        parser.add_option_group(group)
        logger.info('added option group')

    def validate_args(self):
        """ Check that ipaddr is mutually exclusive from typeid, cidr and typelist
        """
        logger.info('validating base options')
        super(DeviceReport, self).validate_args()

        logger.info('validating custom options')
        if sum([self.options.ipaddr is not None,
                (self.options.typeid is not None or self.options.cidr is not None),
                self.options.typelist]) > 1:
            self.parser.error('ipaddr, (typeid/cidr), and typelist '
                              'are mutually exclusive, '
                              'choose from only one group.')

    def main(self):
        """ Setup query and run report with default column set
        """
        logger.info('running main')

        if self.options.ipaddr:
            self.data = self.netprofiler.api.devices.get_details(self.options.ipaddr)
        elif self.options.typelist:
            self.data = self.netprofiler.api.devices.get_types()
        else:
            self.data = self.netprofiler.api.devices.get_all(self.options.typeid,
                                                             self.options.cidr)

        self.print_columns()

    def print_columns(self, paginate=None):
        """ Print out data in a nice formatted table

        `paginate` option will insert a new header after that many
        rows have been printed.  Defaults to None (only single header
        output).
        """
        if not self.data:
            print "No data found."
            return

        if self.options.typelist:
            # two columns only
            headers = ['type_id', 'type']
            data = self.data
        elif self.options.ipaddr:
            # single dict
            headers = self.data.keys()
            data = [self.data.values()]
        else:
            # assume objects are uniform and take keys from first one
            headers = self.data[0].keys()
            data = [d.values() for d in self.data]

        Formatter.print_table(data, headers, paginate=30)


if __name__ == '__main__':
    DeviceReport().run()
